@@ -1,9 +1,10 @@
 //
-//  aboutdlg.cxx
+//  aboutdlg.cc
 //
 //  About box handling for Aven.
 //
-//  Copyright (C) 2001, Mark R. Shinwell.
+//  Copyright (C) 2001-2003 Mark R. Shinwell.
+//  Copyright (C) 2001-2003 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,14 +21,14 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
-#include "aboutdlg.h"
-#include "aven.h"
-#include "message.h"
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include "aboutdlg.h"
+#include "aven.h"
+#include "message.h"
+
 BEGIN_EVENT_TABLE(AboutDlg, wxDialog)
 END_EVENT_TABLE()
 
@@ -37,20 +38,17 @@ AboutDlg::AboutDlg(wxWindow* parent) :
     wxBoxSizer* horiz = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* vert = new wxBoxSizer(wxVERTICAL);
 
-#ifdef AVENGL
-    wxStaticText* title = new wxStaticText(this, 502, wxString("Aven "VERSION" (OpenGL enabled)"));
-#else
-    wxStaticText* title = new wxStaticText(this, 502, wxString("Aven "VERSION));
-#endif
+    wxStaticText* title = new wxStaticText(this, 502,
+                                           wxString("Aven "VERSION));
     wxStaticText* purpose = new wxStaticText(this, 505,
-	wxString(msg(/*Visualisation of Survex 3D files*/209)));
+	wxString(msg(/*Survey visualisation tool*/209)));
     wxStaticText* copyright1 = new wxStaticText(this, 503,
 	    wxString::Format(AVEN_COPYRIGHT_MSG, msg(/*&copy;*/0)));
     wxStaticText* copyright2 = new wxStaticText(this, 504,
 	    wxString::Format(COPYRIGHT_MSG, msg(/*&copy;*/0)));
 
     wxString licence_str;
-    wxString l(msg(/*This is free software.  Aven is licenced under the terms of the GNU General Public Licence version 2, or (at your option) any later version.*/200));
+    wxString l(msg(/*This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public Licence as published by the Free Software Foundation; either version 2 of the Licence, or (at your option) any later version.*/219));
     wxClientDC dc(this);
     dc.SetFont(this->GetFont());
     do {
@@ -90,9 +88,11 @@ AboutDlg::AboutDlg(wxWindow* parent) :
     vert->Add(10, 5, 0, wxTOP, 5);
     vert->Add(purpose, 0, wxLEFT | wxRIGHT, 20);
     vert->Add(10, 5, 0, wxTOP, 5);
+
     vert->Add(copyright1, 0, wxLEFT | wxRIGHT, 20);
     vert->Add(copyright2, 0, wxLEFT | wxRIGHT, 20);
     vert->Add(10, 5, 0, wxTOP, 15);
+    
     vert->Add(licence, 0, wxLEFT | wxRIGHT, 20);
     vert->Add(10, 5, 1, wxALIGN_BOTTOM | wxTOP, 5);
 
