@@ -1,9 +1,10 @@
 //
-//  aven.cxx
+//  aven.cc
 //
 //  Main class for Aven.
 //
 //  Copyright (C) 2001, Mark R. Shinwell.
+//  Copyright (C) 2002-2003,2004 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,6 +21,10 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "aven.h"
 #include "mainfrm.h"
 
@@ -44,6 +49,24 @@ Aven::Aven() :
 
 bool Aven::OnInit()
 {
+#ifdef __WXMAC__
+    // Tell wxMac which the About menu item is so it can be put where MacOS
+    // users expect it to be
+    wxApp::s_macAboutMenuItemId = menu_HELP_ABOUT;
+    // wxMac is supposed to remove this magic command line option (which
+    // Finder passes), but the code in 2.4.2 is bogus.  It just decrements
+    // argc rather than copying argv down.  But luckily it also fails to
+    // set argv[argc] to NULL so we can just recalculate argc, then remove
+    // the -psn_* switch ourselves.
+    if (argv[argc]) {
+	argc = 1;
+	while (argv[argc]) ++argc;
+    }
+    if (argc > 1 && strncmp(argv[1], "-psn_", 5) == 0) {
+	--argc;
+	memmove(argv + 1, argv + 2, argc * sizeof(char *));
+    }
+#endif 
     msg_init(argv);
 
     const char *lang = msg_lang2 ? msg_lang2 : msg_lang;
@@ -52,59 +75,62 @@ bool Aven::OnInit()
 	wxLogNull logNo;
 	wxLocale *loc = new wxLocale();
 	loc->AddCatalogLookupPathPrefix(msg_cfgpth());
-	loc->Init(lang, lang, lang, TRUE, TRUE);
+	if (!loc->Init(lang, lang, lang, TRUE, TRUE)) {
+	    if (strcmp(lang, "sk") == 0) {
+	       // As of 2.2.9, wxWindows has cs but not sk - the two languages
+	       // are close, so this makes sense...
+	       loc->Init("cs", "cs", "cs", TRUE, TRUE);
+	    }
+	}
 	// The existence of the wxLocale object is enough - no need to keep a
 	// pointer to it!
     }
+    // Set LC_NUMERIC back to the default - otherwise we break img.c
+    // FIXME: eventually stop doing this so that floating point values are
+    // displayed correctly for the locale...
+    setlocale(LC_NUMERIC, "C");
 
     wxString survey;
+    bool print_and_exit = false;
 
-    /* Want --version and a decent --help output, which cmdline does for us */
-#ifndef USE_WXCMDLINE
+    /* Want --version and a decent --help output, which cmdline does for us.
+     * wxCmdLine is much less good.
+     */
     static const struct option long_opts[] = {
 	/* const char *name; int has_arg (0 no_argument, 1 required_*, 2 optional_*); int *flag; int val; */
 	{"survey", required_argument, 0, 's'},
+	{"print", no_argument, 0, 'p'},
 	{"help", no_argument, 0, HLP_HELP},
 	{"version", no_argument, 0, HLP_VERSION},
 	{0, 0, 0, 0}
     };
 
-#define short_opts "s:"
+#define short_opts "s:p"
 
     static struct help_msg help[] = {
 	/*			     <-- */
 	{HLP_ENCODELONG(0),          "only load the sub-survey with this prefix"},
+	{HLP_ENCODELONG(1),          "print and exit (requires a 3d file)"},
 	{0, 0}
     };
 
     cmdline_set_syntax_message("[3d file]", NULL);
     cmdline_init(argc, argv, short_opts, long_opts, NULL, help, 0, 1);
-    while (1) {
+    while (true) {
 	int opt = cmdline_getopt();
 	if (opt == EOF) break;
 	if (opt == 's') {
 	    survey = optarg;
 	}
+	if (opt == 'p') {
+	    print_and_exit = true;
+	}
     }
-#else
-    static wxCmdLineEntryDesc cmdline[] = {
-	// FIXME - if this code is ever reenabled, check this is correct, and also handle it below...
-	{ wxCMD_LINE_OPTION, "s", "survey", "only load the sub-survey with this prefix", wxCMD_LINE_VAL_STRING},
-	{ wxCMD_LINE_OPTION, "h", "help", msgPerm(/*Display command line options*/201) },
-	{ wxCMD_LINE_PARAM,  NULL, NULL, msgPerm(/*3d file*/119), wxCMD_LINE_VAL_STRING,
-	  wxCMD_LINE_PARAM_OPTIONAL },
-	{ wxCMD_LINE_NONE }
-    };
 
-    wxCmdLineParser cli(cmdline, argc, argv);
-    int c = cli.Parse();
-    if (c != 0 || cli.Found("h")) {
-	// write in two goes to avoid msg() overwriting its buffer on 2nd call
-	fprintf(stderr, "%s: %s ", msg(/*Syntax*/49), argv[0]);
-	fprintf(stderr, "[%s]\n", msg(/*3d file*/119));
-	exit(c > 0 ? 1 /* syntax error */ : 0 /* --help */);
+    if (print_and_exit && !argv[optind]) {
+	cmdline_syntax(); // FIXME : not a helpful error...
+	exit(1);
     }
-#endif
 
     wxImage::AddHandler(new wxPNGHandler);
 
@@ -125,24 +151,26 @@ bool Aven::OnInit()
     m_Frame = new MainFrm("Aven", wxPoint(50, 50), wxSize(our_width, our_height));
 
 #ifdef _WIN32
-    bool delay = false;
+    const bool delay = false;
 #else
-    bool delay = true;
+    const bool delay = true;
 #endif
 
-#ifndef USE_WXCMDLINE
     if (argv[optind]) {
 	m_Frame->OpenFile(wxString(argv[optind]), survey, delay);
     }
-#else
-    if (cli.GetParamCount() == 1) {
-	wxString file = cli.GetParam(0);
-	m_Frame->OpenFile(file, survey, delay);
+
+    if (print_and_exit) {
+	wxCommandEvent dummy;
+	m_Frame->OnPrint(dummy);
+	m_Frame->OnQuit(dummy);
+	return true;
     }
-#endif
 
     m_Frame->Show(true);
-
+#ifdef _WIN32
+    m_Frame->SetFocus();
+#endif
     return true;
 }
 
