@@ -1,6 +1,6 @@
 /* cavern.c
  * SURVEX Cave surveying software: data reduction main and related functions
- * Copyright (C) 1991-2001 Olly Betts
+ * Copyright (C) 1991-2003,2004 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <config.h>
 #endif
 
+#include <limits.h>
 #include <time.h>
 
 #include "cavern.h"
@@ -40,11 +41,14 @@
 #include "str.h"
 #include "validate.h"
 
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
 # if OS != RISCOS
 /* include header for getcwd() */
-#  if OS == MSDOS && defined(__TURBOC__)
+#  if OS == MSDOS && !defined(__DJGPP__)
 #   include <dir.h>
+#  elif OS == WIN32
+#   include <direct.h>
+#   include <conio.h> /* for _kbhit() and _getch() */
 #  else
 #   include <unistd.h>
 #  endif
@@ -103,14 +107,14 @@ static const struct option long_opts[] = {
    {"no-percentage", no_argument, 0, 0},
 #else
    {"percentage", no_argument, 0, 'p'},
-   {"no-percentage", no_argument, (int*)&fPercent, 0},
+   {"no-percentage", no_argument, 0, 3},
 #endif
    {"output", required_argument, 0, 'o'},
    {"quiet", no_argument, 0, 'q'},
    {"no-auxiliary-files", no_argument, 0, 's'},
    {"warnings-are-errors", no_argument, 0, 'w'},
    {"log", no_argument, 0, 1},
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
    {"chasm-format", no_argument, 0, 'x'},
 #endif
 #if (OS==WIN32)
@@ -121,7 +125,7 @@ static const struct option long_opts[] = {
    {0, 0, 0, 0}
 };
 
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
 #define short_opts "pxao:qswz:"
 #else
 #define short_opts "pao:qswz:"
@@ -136,7 +140,7 @@ static struct help_msg help[] = {
    {HLP_ENCODELONG(4),          "do not create .err file"},
    {HLP_ENCODELONG(5),          "turn warnings into errors"},
    {HLP_ENCODELONG(6),          "log output to .log file"},
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
    {HLP_ENCODELONG(7),          "output data in chasm's 3dx format"},
 #endif
  /*{'z',                        "set optimizations for network reduction"},*/
@@ -164,12 +168,8 @@ extern CDECL int
 main(int argc, char **argv)
 {
    int d;
-   static clock_t tmCPUStart;
-   static time_t tmUserStart;
-   static double tmCPU, tmUser;
-
-   tmUserStart = time(NULL);
-   tmCPUStart = clock();
+   time_t tmUserStart = time(NULL);
+   clock_t tmCPUStart = clock();
    init_screen();
 
    msg_init(argv);
@@ -195,8 +195,8 @@ main(int argc, char **argv)
    totadj = total = totplan = totvert = 0.0;
 
    for (d = 0; d <= 2; d++) {
-      min[d] = REAL_BIG;
-      max[d] = -REAL_BIG;
+      min[d] = HUGE_REAL;
+      max[d] = -HUGE_REAL;
       pfxHi[d] = pfxLo[d] = NULL;
    }
 
@@ -211,6 +211,11 @@ main(int argc, char **argv)
 	 fPercent = 1;
 #endif
 	 break;
+#ifndef NO_PERCENTAGE
+       case 3:
+	 fPercent = 0;
+	 break;
+#endif
        case 'o': {
 	 /* can be a directory (in which case use basename of leaf input)
 	  * or a file (in which case just trim the extension off) */
@@ -225,7 +230,7 @@ main(int argc, char **argv)
 	 }
 	 break;
        }
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
        case 'x': {
 	 fUseNewFormat = 1;
 	 break;
@@ -308,7 +313,7 @@ main(int argc, char **argv)
 
       /* Select defaults settings */
       default_all(pcs);
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       /* we need to get the filename of the first one for our base_source */
       /* and also run_file */
       if (fUseNewFormat) {
@@ -334,12 +339,12 @@ main(int argc, char **argv)
    solve_network(/*stnlist*/); /* Find coordinates of all points */
    validate();
 
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
    if (fUseNewFormat) {
       /* this actually does all the writing */
       if (!cave_close(pimg)) {
 	 char *fnm = add_ext(fnm_output_base, EXT_SVX_3DX);
-	 fatalerror_in_file(fnm, 0, /*Error writing to file*/111);
+	 fatalerror(/*Error writing to file `%s'*/110, fnm);
       }
    } else {
 #endif
@@ -348,7 +353,7 @@ main(int argc, char **argv)
 	 char *fnm = add_ext(fnm_output_base, EXT_SVX_3D);
 	 fatalerror(img_error(), fnm);
       }
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
    }
 #endif
    if (fhErrStat) safe_fclose(fhErrStat);
@@ -358,15 +363,26 @@ main(int argc, char **argv)
    if (!fQuiet) {
       /* clock() typically wraps after 72 minutes, but there doesn't seem
        * to be a better way.  Still 72 minutes means some cave!
+       * We detect if clock() could have wrapped and suppress CPU time
+       * printing in this case.
        */
-      tmCPU = (clock_t)(clock() - tmCPUStart) / (double)CLOCKS_PER_SEC;
-      tmUser = difftime(time(NULL), tmUserStart);
+      double tmUser = difftime(time(NULL), tmUserStart);
+      double tmCPU;
+      clock_t now = clock();
+#define CLOCK_T_WRAP \
+	(sizeof(clock_t)<sizeof(long)?(1ul << (CHAR_BIT * sizeof(clock_t))):0)
+      tmCPU = (now - (unsigned long)tmCPUStart)
+	 / (double)CLOCKS_PER_SEC;
+      if (now < tmCPUStart)
+	 tmCPU += CLOCK_T_WRAP / (double)CLOCKS_PER_SEC;
+      if (tmUser >= tmCPU + CLOCK_T_WRAP / (double)CLOCKS_PER_SEC)
+	 tmCPU = 0;
 
-      /* tmCPU is integer, tmUser not - equivalent to (ceil(tmCPU) >= tmUser) */
+      /* tmUser is integer, tmCPU not - equivalent to (ceil(tmCPU) >= tmUser) */
       if (tmCPU + 1 > tmUser) {
 	 printf(msg(/*CPU time used %5.2fs*/140), tmCPU);
       } else if (tmCPU == 0) {
-	 if (tmUser == 0.0) {
+	 if (tmUser != 0.0) {
 	    printf(msg(/*Time used %5.2fs*/141), tmUser);
 	 } else {
 	    fputs(msg(/*Time used unavailable*/142), stdout);
@@ -446,12 +462,15 @@ do_stats(void)
 	  totvert);
    putnl();
 
-   do_range(2, /*Vertical range = %4.2fm (from */135,
-	    /* at %4.2fm to */136, /* at %4.2fm)*/137);
-   do_range(1, /*North-South range = %4.2fm (from */148,
-	    /* at %4.2fm to */196, /* at %4.2fm)*/197);
-   do_range(0, /*East-West range = %4.2fm (from */149,
-	    /* at %4.2fm to */196, /* at %4.2fm)*/197);
+   /* If there's no underground survey, we've no ranges */
+   if (pfxHi[0]) {
+      do_range(2, /*Vertical range = %4.2fm (from */135,
+	       /* at %4.2fm to */136, /* at %4.2fm)*/137);
+      do_range(1, /*North-South range = %4.2fm (from */148,
+	       /* at %4.2fm to */196, /* at %4.2fm)*/197);
+      do_range(0, /*East-West range = %4.2fm (from */149,
+	       /* at %4.2fm to */196, /* at %4.2fm)*/197);
+   }
 
    print_node_stats();
    /* Also, could give:
