@@ -1,6 +1,6 @@
 /* cmdline.c
  * Wrapper for GNU getopt which deals with standard options
- * Copyright (C) 1998-2001 Olly Betts
+ * Copyright (C) 1998-2001,2003 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,8 @@
 #include <string.h>
 #include <limits.h>
 
+#include "getopt.h"
+
 #include "cmdline.h"
 #include "filename.h"
 
@@ -167,7 +169,7 @@ static void
 syntax_and_help_pointer(void)
 {
    cmdline_syntax();
-   fprintf(stderr, msg(/*Try `%s --help' for more information.\n*/157),
+   fprintf(stderr, msg(/*Try `%s --help' for more information.&#10;*/157),
 	   msg_appname());
    exit(1);
 }
