@@ -4,7 +4,7 @@
 //  Core drawing code for Aven.
 //
 //  Copyright (C) 2000-2001, Mark R. Shinwell.
-//  Copyright (C) 2001-2002 Olly Betts
+//  Copyright (C) 2001-2002,2004 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -30,33 +30,12 @@
 #include "wx.h"
 #include <utility>
 #include <list>
-#ifdef AVENGL
-#ifndef wxUSE_GLCANVAS
-#define wxUSE_GLCANVAS
-#endif
-#include <wx/glcanvas.h>
-#endif
 
 using std::list;
 
 class MainFrm;
 class PointInfo;
 
-#define SCALE_ALL 0
-#define SCALE_NAMES 1
-#define SCALE_CROSSES 2
-#define SCALE_LEGS 3
-#define SCALE_SPECIALPTS 4
-#define SCALE_HIGHLIGHTEDPTS 5
-
-#ifdef AVENGL
-struct Double3 {
-    Double x;
-    Double y;
-    Double z;
-};
-#endif
-
 struct ColourTriple {
     // RGB triple: values are from 0-255 inclusive for each component.
     int r;
@@ -65,7 +44,7 @@ struct ColourTriple {
 };
 
 // Colours for drawing.
-// These must be in the same order as the entries in the array COLOURS in gfxcore.cc.
+// These must be in the same order as the entries in COLOURS[] in gfxcore.cc.
 enum AvenColour {
     col_BLACK = 0,
     col_GREY,
@@ -79,30 +58,22 @@ enum AvenColour {
     col_INDICATOR_2,
     col_YELLOW,
     col_RED,
-    col_CYAN,
     col_LAST // must be the last entry here
 };
 
-class SpecialPoint {
+class Point {
     friend class GfxCore;
-
     Double x, y, z;
-    AvenColour colour;
-    int size;
-#ifndef AVENGL
-    int screen_x, screen_y;
-#endif
+public:
+    Point() {}
+    Point(Double x_, Double y_, Double z_) : x(x_), y(y_), z(z_) {}
 };
 
 class LabelInfo;
 
 extern const ColourTriple COLOURS[]; // defined in gfxcore.cc
 
-#ifdef AVENGL
-class GfxCore : public wxGLCanvas {
-#else
 class GfxCore : public wxWindow {
-#endif
     struct params {
 	Quaternion rotation;
 	Double scale;
@@ -111,56 +82,8 @@ class GfxCore : public wxWindow {
 	    Double y;
 	    Double z;
 	} translation;
-	struct {
-	    int x;
-	    int y;
-	    int z;
-	} display_shift;
     } m_Params;
 
-#ifdef AVENPRES
-    struct PresData {
-	struct {
-	    Double x, y, z;
-	} translation;
-	struct {
-	    int x, y, z;
-	} display_shift;
-	Double scale;
-	Double pan_angle;
-	Double tilt_angle;
-	bool solid_surface;
-    };
-#endif
-
-#ifdef AVENGL
-    struct {
-	// OpenGL display lists.
-	GLint survey; // all underground data
-	GLint surface; // all surface data in uniform colour
-	GLint surface_depth; // all surface data in depth bands
-	GLint grid; // the grid
-	GLint terrain; // surface terrain
-	GLint flat_terrain; // flat surface terrain - used for drape effect
-	GLint map; // map overlay
-    } m_Lists;
-
-    bool m_AntiAlias;
-#endif
-
-    enum LabelFlags {
-	label_NOT_PLOTTED,
-	label_PLOTTED,
-	label_CHECK_AGAIN
-    };
-
-    enum HighlightFlags {
-	hl_NONE = 0,
-	hl_ENTRANCE = 1,
-	hl_EXPORTED = 2,
-	hl_FIXED = 4
-    };
-
     enum LockFlags {
 	lock_NONE = 0,
 	lock_X = 1,
@@ -172,22 +95,9 @@ class GfxCore : public wxWindow {
 	lock_XY = lock_X | lock_Y
     };
 
-    struct {
-#ifdef AVENGL
-	// For the OpenGL version we store the real (x, y, z) coordinates of
-	// each station.
-	Double3* vertices;
-#else
-	// For the non-OpenGL version we store the screen coordinates of
-	// each station after the transformation has been applied.
-	wxPoint* vertices;
-#endif
-	int* num_segs;
-    } m_CrossData;
-
     struct PlotData {
-	wxPoint* vertices;
-	wxPoint* surface_vertices;
+	Point *vertices;
+	Point *surface_vertices;
 	int* num_segs;
 	int* surface_num_segs;
     };
@@ -200,45 +110,8 @@ class GfxCore : public wxWindow {
 	int drag_start_offset_y;
     } m_ScaleBar;
 
-    struct HighlightedPt {
-	int x;
-	int y;
-	HighlightFlags flags;
-    };
-
-#ifdef AVENGL
-    struct {
-	// Viewing volume parameters: these are all negative!
-	Double left;
-	Double bottom;
-	Double nearface;
-    } m_Volume;
-
-    struct {
-	GLuint surface;
-	GLuint map;
-    } m_Textures;
-
-    bool m_SolidSurface;
-#endif
-
-    Double floor_alt;
-    bool terrain_rising;
-
-    struct GridPointInfo {
-	long x, y; // screen coordinates
-	LabelInfo* label;
-    };
-
-    list<SpecialPoint> m_SpecialPoints;
-#ifdef AVENPRES
-    list<pair<PresData, Quaternion> > m_Presentation;
-    list<pair<PresData, Quaternion> >::iterator m_PresIterator;
-#endif
     double m_MaxExtent; // twice the maximum of the {x,y,z}-extents, in survey coordinates.
-    LabelFlags* m_LabelGrid;
-    HighlightedPt* m_HighlightedPts;
-    int m_NumHighlightedPts;
+    char *m_LabelGrid;
     bool m_RotationOK;
     LockFlags m_Lock;
     Matrix4 m_RotationMatrix;
@@ -257,14 +130,12 @@ class GfxCore : public wxWindow {
     int* m_SurfacePolylines;
     int m_Bands;
     Double m_InitialScale;
-    bool m_FreeRotMode;
     Double m_TiltAngle;
     Double m_PanAngle;
     bool m_Rotating;
     Double m_RotationStep;
+    int m_SwitchingTo;
     bool m_ReverseControls;
-    bool m_SwitchingToPlan;
-    bool m_SwitchingToElevation;
     bool m_Crosses;
     bool m_Legs;
     bool m_Names;
@@ -272,19 +143,13 @@ class GfxCore : public wxWindow {
     wxFont m_Font;
     bool m_Depthbar;
     bool m_OverlappingNames;
-    bool m_LabelCacheNotInvalidated;
-    LabelFlags* m_LabelsLastPlotted;
-    wxRect m_LabelCacheExtend;
     bool m_Compass;
     bool m_Clino;
     int m_XSize;
     int m_YSize;
     int m_XCentre;
     int m_YCentre;
-    wxPoint m_LabelShift;
-    wxTimer m_Timer;
     PlotData* m_PlotData;
-    LabelInfo** m_Labels;
     bool m_InitialisePending;
     enum { drag_NONE, drag_MAIN, drag_COMPASS, drag_ELEV, drag_SCALE } m_LastDrag;
     bool m_MouseOutsideCompass;
@@ -298,57 +163,22 @@ class GfxCore : public wxWindow {
     bool m_FixedPts;
     bool m_ExportedPts;
     bool m_Grid;
-    int m_NumCrosses;
-    list<GridPointInfo>* m_PointGrid;
+
+    list<LabelInfo*> *m_PointGrid;
     bool m_HitTestGridValid;
-    bool m_TerrainLoaded;
-    list<PointInfo> m_PointCache;
 
-#ifdef AVENPRES
-    int m_DoingPresStep;
-    struct step_params {
-	Double pan_angle;
-	Double tilt_angle;
-	Quaternion rotation;
-	Double scale;
-	struct {
-	    Double x;
-	    Double y;
-	    Double z;
-	} translation;
-	struct {
-	    int x;
-	    int y;
-	    int z;
-	} display_shift;
-    };
+    Point m_here, m_there;
 
-    struct {
-	step_params from;
-	step_params to;
-    } m_PresStep;
-#endif
+    wxStopWatch timer;
+    long drawtime;
+    
+    bool clipping;
 
-    struct {
-	Double x, y, z;
-    } m_here;
-
-    struct {
-	Double x, y, z;
-    } m_there;
-
-#ifndef AVENGL
     wxPen* m_Pens;
     wxBrush* m_Brushes;
-#endif
 
     void SetColour(AvenColour col, bool background = false /* true => foreground */) {
 	assert(col >= (AvenColour) 0 && col < col_LAST);
-#ifdef AVENGL
-	glColor3f(GLfloat(COLOURS[col].r) / 256.0,
-		  GLfloat(COLOURS[col].g) / 256.0,
-		  GLfloat(COLOURS[col].b) / 256.0);
-#else
 	if (background) {
 	    assert(m_Brushes[col].Ok());
 	    m_DrawDC.SetBrush(m_Brushes[col]);
@@ -357,21 +187,8 @@ class GfxCore : public wxWindow {
 	    assert(m_Pens[col].Ok());
 	    m_DrawDC.SetPen(m_Pens[col]);
 	}
-#endif
     }
 
-#ifdef AVENGL
-    void SetGLProjection();
-    void SetModellingTransformation();
-    void ClearBackgroundAndBuffers();
-    void SetGLAntiAliasing();
-    void CheckGLError(const wxString& where);
-    void SetTerrainColour(Double);
-    void LoadTexture(const wxString& file, GLuint* texture);
-    void RenderMap();
-    void SetSolidSurface(bool);
-#endif
-
     Double XToScreen(Double x, Double y, Double z) {
 	return Double(x * m_RotationMatrix.get(0, 0) +
 		      y * m_RotationMatrix.get(0, 1) +
@@ -389,9 +206,15 @@ class GfxCore : public wxWindow {
 		      y * m_RotationMatrix.get(2, 1) +
 		      z * m_RotationMatrix.get(2, 2));
     }
-
+    
     Double GridXToScreen(Double x, Double y, Double z);
     Double GridYToScreen(Double x, Double y, Double z);
+    Double GridXToScreen(const Point &p) {
+	return GridXToScreen(p.x, p.y, p.z);
+    }
+    Double GridYToScreen(const Point &p) {
+	return GridYToScreen(p.x, p.y, p.z);
+    }
 
     void CheckHitTestGrid(wxPoint& point, bool centre);
 
@@ -400,8 +223,11 @@ class GfxCore : public wxWindow {
     void DrawTick(wxCoord cx, wxCoord cy, int angle_cw);
     wxString FormatLength(Double, bool scalebar = true);
 
-    void SetScale(Double scale, int what = SCALE_ALL);
-    void SetScaleInitial(Double scale, int what = SCALE_ALL);
+    void SetScale(Double scale);
+    void SetScaleInitial(Double scale);
+    void DrawBand(int num_polylines, const int *num_segs, const Point *vertices,
+		  Double m_00, Double m_01, Double m_02,
+		  Double m_20, Double m_21, Double m_22);
     void RedrawOffscreen();
     void TryToFreeArrays();
     void FirstShow();
@@ -419,30 +245,19 @@ class GfxCore : public wxWindow {
     void HandleTilt(wxPoint);
     void HandleTranslate(wxPoint);
 
+    void TranslateCave(int dx, int dy);
     void TiltCave(Double tilt_angle);
     void TurnCave(Double angle);
     void TurnCaveTo(Double angle);
 
-    void StartTimer();
-    void StopTimer();
-
     void DrawNames();
     void NattyDrawNames();
     void SimpleDrawNames();
 
-    void Defaults();
-    void DefaultParameters();
-
     void Repaint();
 
-#ifdef AVENPRES
-    void PresGoto(PresData& d, Quaternion& q);
-#endif
-
     void CreateHitTestGrid();
 
-    void RenderTerrain(Double floor_alt);
-
 public:
     bool m_Degrees;
     bool m_Metric;
@@ -454,10 +269,11 @@ public:
     void InitialiseOnNextResize() { m_InitialisePending = true; }
     void InitialiseTerrain();
 
-    void ClearSpecialPoints();
-    void AddSpecialPoint(Double x, Double y, Double z);
-    void DisplaySpecialPoints();
+    void ForceRefresh();
 
+    void RefreshLine(const Point &a1, const Point &b1,
+		     const Point &a2, const Point &b2);
+ 
     void SetHere();
     void SetHere(Double x, Double y, Double z);
     void SetThere();
@@ -465,17 +281,6 @@ public:
 
     void CentreOn(Double x, Double y, Double z);
 
-#ifdef AVENPRES
-    void RecordPres(FILE* fp);
-    void LoadPres(FILE* fp);
-    void PresGo();
-    void PresGoBack();
-    void RestartPres();
-
-    bool AtStartOfPres();
-    bool AtEndOfPres();
-#endif
-
     void OnDefaults();
     void OnPlan();
     void OnElevation();
@@ -515,10 +320,6 @@ public:
     void OnShowEntrances();
     void OnShowFixedPts();
     void OnShowExportedPts();
-#ifdef AVENGL
-    void OnAntiAlias();
-    void OnSolidSurface();
-#endif
     void OnCancelDistLine();
 
     void OnPaint(wxPaintEvent&);
@@ -530,7 +331,8 @@ public:
     void OnRButtonDown(wxMouseEvent& event);
     void OnRButtonUp(wxMouseEvent& event);
     void OnSize(wxSizeEvent& event);
-    void OnTimer(wxIdleEvent& event);
+    void OnIdle(wxIdleEvent& event);
+    bool Animate();
 
     void OnKeyPress(wxKeyEvent &e);
 
@@ -573,10 +375,6 @@ public:
     void OnShowEntrancesUpdate(wxUpdateUIEvent&);
     void OnShowExportedPtsUpdate(wxUpdateUIEvent&);
     void OnShowFixedPtsUpdate(wxUpdateUIEvent&);
-#ifdef AVENGL
-    void OnAntiAliasUpdate(wxUpdateUIEvent&);
-    void OnSolidSurfaceUpdate(wxUpdateUIEvent&);
-#endif
     void OnIndicatorsUpdate(wxUpdateUIEvent&);
     void OnCancelDistLineUpdate(wxUpdateUIEvent&);
 
@@ -586,6 +384,9 @@ public:
     void OnToggleDegrees();
     void OnToggleDegreesUpdate(wxUpdateUIEvent& cmd);
 
+    void OnPrint(const wxString &filename, const wxString &title,
+		 const wxString &datestamp);
+
 private:
     DECLARE_EVENT_TABLE()
 };
