@@ -1,6 +1,6 @@
 /* img.c
  * Routines for reading and writing Survex ".3d" image files
- * Copyright (C) 1993-2002 Olly Betts
+ * Copyright (C) 1993-2004 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +42,9 @@
 # define TIMENA "Time not available."
 # define TIMEFMT "%a,%Y.%m.%d %H:%M:%S %Z"
 # define EXT_SVX_3D "3d"
+# define EXT_SVX_POS "pos"
+# define FNM_SEP_EXT '.'
+# define METRES_PER_FOOT 0.3048 /* exact value */
 # define xosmalloc(L) malloc((L))
 # define xosrealloc(L,S) realloc((L),(S))
 # define osfree(P) free((P))
@@ -59,7 +62,7 @@
  * to work as intended
  */
 # define fputsnl(S, FH) do {fputs((S), (FH)); putc('\n', (FH));} while(0)
-# define ASSERT(X)
+# define SVX_ASSERT(X)
 
 static long
 get32(FILE *fh)
@@ -80,10 +83,47 @@ put32(long w, FILE *fh)
    putc((char)(w >> 24l), fh);
 }
 
+static short
+get16(FILE *fh)
+{
+   short w = getc(fh);
+   w |= (long)getc(fh) << 8l;
+   return w;
+}
+
+static void
+put16(short w, FILE *fh)
+{
+   putc((char)(w), fh);
+   putc((char)(w >> 8l), fh);
+}
+
+static char *
+baseleaf_from_fnm(const char *fnm)
+{
+   const char *p;
+   char *q;
+   size_t len;
+
+   p = fnm;
+   q = strrchr(p, '/');
+   if (q) p = q + 1;
+   q = strrchr(p, '\\');
+   if (q) p = q + 1;
+
+   q = strrchr(p, FNM_SEP_EXT);
+   if (q) len = (const char *)q - p; else len = strlen(p);
+
+   q = xosmalloc(len + 1);
+   if (!q) return NULL;
+   memcpy(q, p, len);
+   q[len] = '\0';
+   return q;
+}
 #endif
 
+#include <math.h>
 #ifdef HAVE_ROUND
-# include <math.h>
 extern double round(double); /* prototype is often missing... */
 # define my_round round
 #else
@@ -100,7 +140,7 @@ my_round(double x) {
 #else
 /* What about top bit set chars? */
 int my_strcasecmp(const char *s1, const char *s2) {
-   register c1, c2;
+   register int c1, c2;
    do {
       c1 = *s1++;
       c2 = *s2++;
@@ -131,6 +171,7 @@ static img_errcode img_errno = IMG_NONE;
 #define FILEID "Survex 3D Image File"
 
 #define EXT_PLT "plt"
+#define EXT_PLF "plf"
 #define EXT_XYZ "xyz"
 
 /* Attempt to string paste to ensure we are passed a literal string */
@@ -207,12 +248,16 @@ check_label_space(img *pimg, size_t len)
    return fTrue;
 }
 
+#define has_ext(F,L,E) ((L) > LITLEN(E) + 1 &&\
+			(F)[(L) - LITLEN(E) - 1] == FNM_SEP_EXT &&\
+			my_strcasecmp((F) + (L) - LITLEN(E), E) == 0)
+
 img *
 img_open_survey(const char *fnm, const char *survey)
 {
    img *pimg;
    size_t len;
-   char buf[LITLEN(FILEID) + 1];
+   char buf[LITLEN(FILEID) + 9];
    int ch;
 
    if (fDirectory(fnm)) {
@@ -236,8 +281,8 @@ img_open_survey(const char *fnm, const char *survey)
 
    pimg->fh = fopenWithPthAndExt("", fnm, EXT_SVX_3D, "rb", NULL);
    if (pimg->fh == NULL) {
-      osfree(pimg);
       osfree(pimg->label_buf);
+      osfree(pimg);
       img_errno = IMG_FILENOTFOUND;
       return NULL;
    }
@@ -256,6 +301,7 @@ img_open_survey(const char *fnm, const char *survey)
 
    pimg->survey = NULL;
    pimg->survey_len = 0;
+   pimg->separator = '.';
 
    pimg->title = pimg->datestamp = NULL;
    if (survey) {
@@ -264,7 +310,11 @@ img_open_survey(const char *fnm, const char *survey)
 	 if (survey[len - 1] == '.') len--;
 	 if (len) {
 	    char *p;
-	    pimg->survey = osmalloc(len + 2);
+	    pimg->survey = xosmalloc(len + 2);
+	    if (!pimg->survey) {
+	       img_errno = IMG_OUTOFMEMORY;
+	       goto error;
+	    }
 	    memcpy(pimg->survey, survey, len);
 	    /* Set title to leaf survey name */
 	    pimg->survey[len] = '\0';
@@ -290,9 +340,7 @@ img_open_survey(const char *fnm, const char *survey)
    pimg->pending = 0;
 
    len = strlen(fnm);
-   if (len > LITLEN(EXT_SVX_POS) + 1 &&
-       fnm[len - LITLEN(EXT_SVX_POS) - 1] == FNM_SEP_EXT &&
-       my_strcasecmp(fnm + len - LITLEN(EXT_SVX_POS), EXT_SVX_POS) == 0) {
+   if (has_ext(fnm, len, EXT_SVX_POS)) {
 pos_file:
       pimg->version = -1;
       if (!pimg->survey) pimg->title = baseleaf_from_fnm(fnm);
@@ -305,12 +353,13 @@ pos_file:
       return pimg;
    }
 
-   if (len > LITLEN(EXT_PLT) + 1 &&
-       fnm[len - LITLEN(EXT_PLT) - 1] == FNM_SEP_EXT &&
-       my_strcasecmp(fnm + len - LITLEN(EXT_PLT), EXT_PLT) == 0) {
+   if (has_ext(fnm, len, EXT_PLT) || has_ext(fnm, len, EXT_PLF)) {
       long fpos;
 plt_file:
       pimg->version = -2;
+      /* Spaces aren't legal in Compass station names, but dots are, so
+       * use space as the level separator */
+      pimg->separator = ' ';
       pimg->start = 0;
       if (!pimg->survey) pimg->title = baseleaf_from_fnm(fnm);
       pimg->datestamp = my_strdup(TIMENA);
@@ -319,7 +368,7 @@ plt_file:
 	 goto error;
       }
       while (1) {
-	 int ch = getc(pimg->fh);
+	 ch = getc(pimg->fh);
 	 switch (ch) {
 	  case '\x1a':
 	    fseek(pimg->fh, -1, SEEK_CUR);
@@ -376,12 +425,12 @@ plt_file:
       }
    }
 
-   if (len > LITLEN(EXT_XYZ) + 1 &&
-       fnm[len - LITLEN(EXT_XYZ) - 1] == FNM_SEP_EXT &&
-       my_strcasecmp(fnm + len - LITLEN(EXT_XYZ), EXT_XYZ) == 0) {
-      long fpos;
+   if (has_ext(fnm, len, EXT_XYZ)) {
       char *line;
 xyz_file:
+      /* Spaces aren't legal in CMAP station names, but dots are, so
+       * use space as the level separator */
+      pimg->separator = ' ';
       line = getline_alloc(pimg->fh);
       if (!line) {
 	 img_errno = IMG_OUTOFMEMORY;
@@ -445,22 +494,26 @@ xyz_file:
    }
 
    if (fread(buf, LITLEN(FILEID) + 1, 1, pimg->fh) != 1 ||
-       memcmp(buf, FILEID"\n", LITLEN(FILEID)) != 0) {
+       memcmp(buf, FILEID"\n", LITLEN(FILEID) + 1) != 0) {
+      if (fread(buf + LITLEN(FILEID) + 1, 8, 1, pimg->fh) == 1 &&
+	  memcmp(buf, FILEID"\r\nv0.01\r\n", LITLEN(FILEID) + 9) == 0) {
+	 /* v0 3d file with DOS EOLs */
+	 pimg->version = 0;
+	 goto v03d;
+      }
+      rewind(pimg->fh);
       if (buf[1] == ' ') {
 	 if (buf[0] == ' ') {
 	    /* Looks like a CMAP .xyz file ... */
-	    rewind(pimg->fh);
 	    goto xyz_file;
 	 } else if (strchr("ZSNF", buf[0])) {
 	    /* Looks like a Compass .plt file ... */
 	    /* Almost certainly it'll start "Z " */
-	    rewind(pimg->fh);
 	    goto plt_file;
 	 }
       }
       if (buf[0] == '(') {
 	 /* Looks like a Survex .pos file ... */
-	 rewind(pimg->fh);
 	 goto pos_file;
       }
       img_errno = IMG_BADFORMAT;
@@ -497,6 +550,7 @@ xyz_file:
       goto error;
    }
 
+v03d:
    if (!pimg->title)
        pimg->title = getline_alloc(pimg->fh);
    else
@@ -573,8 +627,8 @@ img_open_write(const char *fnm, char *title_buf, bool fBinary)
 
    pimg->fh = fopen(fnm, "wb");
    if (!pimg->fh) {
-      osfree(pimg);
       osfree(pimg->label_buf);
+      osfree(pimg);
       img_errno = IMG_CANTOPENOUT;
       return NULL;
    }
@@ -654,8 +708,8 @@ subtract_xyz_shot_deltas(img_point *pt, const char *line)
 static int
 read_coord(FILE *fh, img_point *pt)
 {
-   ASSERT(fh);
-   ASSERT(pt);
+   SVX_ASSERT(fh);
+   SVX_ASSERT(pt);
    pt->x = get32(fh) / 100.0;
    pt->y = get32(fh) / 100.0;
    pt->z = get32(fh) / 100.0;
@@ -703,12 +757,13 @@ img_read_item(img *pimg, img_point *p)
 	 if (opt < 15) {
 	    /* 1-14 mean trim that many levels from current prefix */
 	    int c;
-	    if (pimg->label_len <= 16) {
+	    if (pimg->label_len <= 17) {
 	       /* zero prefix using "0" */
 	       img_errno = IMG_BADFORMAT;
 	       return img_BAD;
 	    }
-	    c = pimg->label_len - 16 - 1;
+	    /* extra - 1 because label_len points to one past the end */
+	    c = pimg->label_len - 17 - 1;
 	    while (pimg->label_buf[c] != '.' || --opt > 0) {
 	       if (--c < 0) {
 		  /* zero prefix using "0" */
@@ -915,6 +970,7 @@ img_read_item(img *pimg, img_point *p)
 	    img_errno = feof(pimg->fh) ? IMG_BADFORMAT : IMG_READERROR;
 	    return img_BAD;
 	 }
+	 pimg->label_buf[len] = '\0';
 	 break;
        }
        case 4:
@@ -1077,7 +1133,6 @@ img_read_item(img *pimg, img_point *p)
 
       pimg->label_buf[0] = '\0';
       while (!feof(pimg->fh)) {
-	 char *b;
 	 if (!fgets(pimg->label_buf + off, pimg->buf_len - off, pimg->fh)) {
 	    img_errno = IMG_READERROR;
 	    return img_BAD;
@@ -1149,8 +1204,7 @@ skip_to_N:
 		  return img_BAD;
 	       }
 	       while (line[len] > 32) ++len;
-	       if (pimg->survey && pimg->label_len == 0)
-		  pimg->pending = -1;
+	       if (pimg->label_len == 0) pimg->pending = -1;
 	       if (!check_label_space(pimg, len + 1)) {
 		  osfree(line);
 		  img_errno = IMG_OUTOFMEMORY;
@@ -1172,10 +1226,17 @@ skip_to_N:
 		  goto skip_to_N;
 	       }
 	       if (ch == 'D' && pimg->pending == -1) {
-		  fpos = ftell(pimg->fh) - 1;
-		  fseek(pimg->fh, pimg->start, SEEK_SET);
-		  ch = getc(pimg->fh);
-		  pimg->pending = 0;
+		  if (pimg->survey) {
+		     fpos = ftell(pimg->fh) - 1;
+		     fseek(pimg->fh, pimg->start, SEEK_SET);
+		     ch = getc(pimg->fh);
+		     pimg->pending = 0;
+		  } else {
+		     /* If a file actually has a 'D' before any 'M', then
+		      * pretend the 'D' is an 'M' - one of the examples
+		      * in the docs was like this! */
+		     ch = 'M';
+		  }
 	       }
 	       line = getline_alloc(pimg->fh);
 	       if (!line) {
@@ -1192,9 +1253,9 @@ skip_to_N:
 		  }
 		  return img_BAD;
 	       }
-	       p->x /= METRES_PER_FOOT;
-	       p->y /= METRES_PER_FOOT;
-	       p->z /= METRES_PER_FOOT;
+	       p->x *= METRES_PER_FOOT;
+	       p->y *= METRES_PER_FOOT;
+	       p->z *= METRES_PER_FOOT;
 	       q = strchr(line, 'S');
 	       if (!q) {
 		  osfree(line);
@@ -1203,22 +1264,20 @@ skip_to_N:
 	       }
 	       ++q;
 	       len = 0;
-	       while (q[len] > ' ') {
-		  /* change dots to spaces.  spaces aren't legal in compass
-		   * station names, while dots are the survey level separator
-		   * in Survex */
-		  if (q[len] == '.') q[len] = ' ';
-		  ++len;
-	       }
+	       while (q[len] > ' ') ++len;
 	       q[len] = '\0';
-	       len += 2; /* '.' and '\0' */
+	       len += 2; /* ' ' and '\0' */
 	       if (!check_label_space(pimg, pimg->label_len + len)) {
 		  img_errno = IMG_OUTOFMEMORY;
 		  return img_BAD;
 	       }
 	       pimg->label = pimg->label_buf;
-	       pimg->label[pimg->label_len] = '.';
-	       memcpy(pimg->label + pimg->label_len + 1, q, len - 1);
+	       if (pimg->label_len) {
+		   pimg->label[pimg->label_len] = ' ';
+		   memcpy(pimg->label + pimg->label_len + 1, q, len - 1);
+	       } else {
+		   memcpy(pimg->label, q, len - 1);
+	       }
 	       osfree(line);
 	       pimg->flags = img_SFLAG_UNDERGROUND; /* default flags */
 	       if (fpos != -1) {
@@ -1242,7 +1301,9 @@ skip_to_N:
       if (pimg->pending) {
 	 /* pending MOVE or LINE or LABEL or STOP */
 	 int r = pimg->pending - 4;
-	 pimg->label = "";
+	 /* Set label to empty - don't use "" as we adjust label relative
+	  * to label_buf when label_buf is reallocated. */
+	 pimg->label = pimg->label_buf + strlen(pimg->label_buf);
 	 pimg->flags = 0;
 	 if (r == img_LABEL) {
 	    /* nasty magic */
@@ -1287,13 +1348,9 @@ skip_to_N:
 	 q = memchr(pimg->label, ' ', 6);
 	 if (!q) q = pimg->label + 6;
 	 *q = '\0';
-	 /* change dots to spaces.  spaces aren't legal in CMAP
-	  * station names, while dots are the survey level separator
-	  * in Survex */
-	 while (--q >= pimg->label) if (*q == '.') *q = ' ';
 
 	 read_xyz_station_coords(p, line);
-	 
+ 
 	 /* FIXME: look at prev for lines (line + 32, 5) */
 	 /* FIXME: duplicate stations... */
 	 return img_LABEL;
@@ -1305,25 +1362,17 @@ skip_to_N:
 	    img_errno = IMG_BADFORMAT;
 	    return img_BAD;
 	 }
-	 
+ 
 	 memcpy(old, line, 7);
 	 q = memchr(old, ' ', 7);
 	 if (!q) q = old + 7;
 	 *q = '\0'; 
-	 /* change dots to spaces.  spaces aren't legal in CMAP
-	  * station names, while dots are the survey level separator
-	  * in Survex */
-	 while (--q > old) if (*q == '.') *q = ' ';
-	 
+
 	 memcpy(new, line + 7, 7);
 	 q = memchr(new, ' ', 7);
 	 if (!q) q = new + 7;
 	 *q = '\0'; 
-	 /* change dots to spaces.  spaces aren't legal in CMAP
-	  * station names, while dots are the survey level separator
-	  * in Survex */
-	 while (--q > new) if (*q == '.') *q = ' ';
-	 
+
 	 pimg->flags = img_SFLAG_UNDERGROUND;
 
 	 if (strcmp(old, new) == 0) {
@@ -1333,7 +1382,7 @@ skip_to_N:
 	    osfree(line);
 	    return img_LABEL;
 	 }
-	 
+ 
 	 if (strcmp(old, pimg->label) == 0) {
 	    pimg->pending = img_LINE + 4;
 	    read_xyz_shot_coords(p, line);
@@ -1364,7 +1413,7 @@ write_v3label(img *pimg, int opt, const char *s)
       if (s[len] == '.') dot = len + 1;
    }
 
-   ASSERT(len <= pimg->label_len);
+   SVX_ASSERT(len <= pimg->label_len);
    n = pimg->label_len - len;
    if (len == 0) {
       if (pimg->label_len) putc(0, pimg->fh);
@@ -1394,7 +1443,7 @@ write_v3label(img *pimg, int opt, const char *s)
       putc(n, pimg->fh);
    } else if (n < 0xffff + 0xfe) {
       putc(0xfe, pimg->fh);
-      put16(n - 0xfe, pimg->fh);
+      put16((short)(n - 0xfe), pimg->fh);
    } else {
       putc(0xff, pimg->fh);
       put32(n, pimg->fh);
@@ -1440,7 +1489,7 @@ img_write_item(img *pimg, int code, int flags, const char *s,
    } else {
       size_t len;
       INT32_T opt = 0;
-      ASSERT(pimg->version > 0);
+      SVX_ASSERT(pimg->version > 0);
       switch (code) {
        case img_LABEL:
 	 if (pimg->version == 1) {
