@@ -4,7 +4,7 @@
 //  Main frame handling for Aven.
 //
 //  Copyright (C) 2000-2001, Mark R. Shinwell.
-//  Copyright (C) 2001-2002 Olly Betts
+//  Copyright (C) 2001-2003,2004 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -25,13 +25,16 @@
 #define mainfrm_h
 
 #include "wx.h"
+#include <wx/docview.h> // for m_FileHistory
+#include <wx/print.h>
+#include <wx/printdlg.h>
 #include "gfxcore.h"
 #include "message.h"
 #include "aventreectrl.h"
 #include "img.h"
 
 #include <list>
-#if 0 // if you turn this back on, reenabled the check in configure.in too
+#if 0 // if you turn this back on, reenable the check in configure.in too
 #ifdef HAVE_EXT_HASH_MAP
 #include <ext/hash_map>
 #elif defined HAVE_HASH_MAP
@@ -56,9 +59,8 @@ extern const int NUM_DEPTH_COLOURS;
 enum {
     menu_FILE_OPEN = 1000,
     menu_FILE_OPEN_PRES,
-#ifdef AVENGL
-    menu_FILE_OPEN_TERRAIN,
-#endif
+    menu_FILE_PRINT,
+    menu_FILE_PAGE_SETUP,
     menu_FILE_QUIT,
     menu_ROTATION_START,
     menu_ROTATION_STOP,
@@ -101,10 +103,6 @@ enum {
     menu_VIEW_STATUS_BAR,
     menu_VIEW_GRID,
     menu_VIEW_SIDE_PANEL,
-#ifdef AVENGL
-    menu_VIEW_ANTIALIAS,
-    menu_VIEW_SOLID_SURFACE,
-#endif
     menu_VIEW_METRIC,
     menu_VIEW_DEGREES,
     menu_PRES_CREATE,
@@ -127,7 +125,6 @@ class PointInfo {
     Double x, y, z;
     bool isLine; // false => move, true => draw line
     bool isSurface;
-    wxPoint* dest;
 
 public:
     Double GetX() const { return x; }
@@ -135,13 +132,21 @@ public:
     Double GetZ() const { return z; }
     bool IsLine() const { return isLine; }
     bool IsSurface() const { return isSurface; }
-
-    void SetDestination(wxPoint* p) { dest = p; }
-    wxPoint* GetDestination() const { return dest; }
 };
 
+#define LFLAG_SURFACE		img_SFLAG_SURFACE
+#define LFLAG_UNDERGROUND	img_SFLAG_UNDERGROUND
+#define LFLAG_EXPORTED		img_SFLAG_EXPORTED
+#define LFLAG_FIXED		img_SFLAG_FIXED
+#define LFLAG_ENTRANCE		0x100
+#define LFLAG_HIGHLIGHTED	0x200
+
+class LabelPlotCmp;
+
 class LabelInfo {
     friend class MainFrm;
+    friend class GfxCore;
+    friend class LabelPlotCmp;
     Double x, y, z;
     wxString text;
     int flags;
@@ -154,15 +159,20 @@ public:
 
     wxString GetText() const { return text; }
 
-    bool IsEntrance() const { return flags & img_SFLAG_ENTRANCE; }
-    bool IsFixedPt() const { return flags & img_SFLAG_FIXED; }
-    bool IsExportedPt() const { return flags & img_SFLAG_EXPORTED; }
-    bool IsUnderground() const { return flags & img_SFLAG_UNDERGROUND; }
-    bool IsSurface() const { return flags & img_SFLAG_SURFACE; }
+    bool IsEntrance() const { return (flags & LFLAG_ENTRANCE) != 0; }
+    bool IsFixedPt() const { return (flags & LFLAG_FIXED) != 0; }
+    bool IsExportedPt() const { return (flags & LFLAG_EXPORTED) != 0; }
+    bool IsUnderground() const { return (flags & LFLAG_UNDERGROUND) != 0; }
+    bool IsSurface() const { return (flags & LFLAG_SURFACE) != 0; }
+    bool IsHighLighted() const { return (flags & LFLAG_HIGHLIGHTED) != 0; }
 };
 
 class MainFrm : public wxFrame {
+    wxFileHistory m_history;
+public: // FIXME: just public for workaround bodge in
+        // wxSplitterWindow::OnSplitterDClick
     int m_SashPosition;
+private:
     list<PointInfo*>* m_Points;
     list<LabelInfo*> m_Labels;
     Double m_XExt;
@@ -197,38 +207,20 @@ class MainFrm : public wxFrame {
     wxStaticText* m_Found;
     wxCheckBox* m_RegexpCheckBox;
     wxString m_File;
-#ifdef AVENPRES
-    FILE* m_PresFP;
-    bool m_PresLoaded;
-    bool m_Recording;
-#endif
+    wxString m_Title, m_DateStamp;
+    int separator; // character separating survey levels (often '.')
 
     struct {
 	Double x, y, z;
     } m_Offsets;
 
-#ifdef AVENGL
-    struct {
-	Double xmin, xmax;
-	Double ymin, ymax;
-	Double zmin, zmax;
-    } m_TerrainExtents;
-
-    struct {
-	int x, y;
-    } m_TerrainSize;
-
-    Double* m_TerrainGrid;
-#endif
+    wxPageSetupData m_pageSetupData;
+    wxPrintData m_printData;
 
     void SetTreeItemColour(LabelInfo* label);
     void FillTree();
     void ClearPointLists();
     bool LoadData(const wxString& file, wxString prefix = "");
-#ifdef AVENGL
-    bool LoadTerrain(const wxString& file);
-    void OpenTerrain(const wxString& file);
-#endif
     void SortIntoDepthBands(list<PointInfo*>& points);
     void IntersectLineWithPlane(Double x0, Double y0, Double z0,
 				Double x1, Double y1, Double z1,
@@ -250,10 +242,8 @@ public:
     MainFrm(const wxString& title, const wxPoint& pos, const wxSize& size);
     ~MainFrm();
 
+    void OnMRUFile(wxCommandEvent& event);
     void OpenFile(const wxString& file, wxString survey = "", bool delay = false);
-#ifdef AVENPRES
-    void OnOpenPresUpdate(wxUpdateUIEvent& event);
-#endif
     void OnFileOpenTerrainUpdate(wxUpdateUIEvent& event);
 
     void OnFind(wxCommandEvent& event);
@@ -261,35 +251,16 @@ public:
 
     void OnOpen(wxCommandEvent& event);
     void OnFileOpenTerrain(wxCommandEvent& event);
-#ifdef AVENPRES
-    void OnOpenPres(wxCommandEvent& event);
-#endif
+    void OnPrint(wxCommandEvent& event);
+    void OnPageSetup(wxCommandEvent& event);
     void OnQuit(wxCommandEvent& event);
 
-#ifdef AVENPRES
-    void OnPresCreate(wxCommandEvent& event);
-    void OnPresGo(wxCommandEvent& event);
-    void OnPresGoBack(wxCommandEvent& event);
-    void OnPresFinish(wxCommandEvent& event);
-    void OnPresRestart(wxCommandEvent& event);
-    void OnPresRecord(wxCommandEvent& event);
-    void OnPresErase(wxCommandEvent& event);
-    void OnPresEraseAll(wxCommandEvent& event);
-
-    void OnPresCreateUpdate(wxUpdateUIEvent& event);
-    void OnPresGoUpdate(wxUpdateUIEvent& event);
-    void OnPresGoBackUpdate(wxUpdateUIEvent& event);
-    void OnPresFinishUpdate(wxUpdateUIEvent& event);
-    void OnPresRestartUpdate(wxUpdateUIEvent& event);
-    void OnPresRecordUpdate(wxUpdateUIEvent& event);
-    void OnPresEraseUpdate(wxUpdateUIEvent& event);
-    void OnPresEraseAllUpdate(wxUpdateUIEvent& event);
-#endif
-
     void OnAbout(wxCommandEvent& event);
     void OnClose(wxCloseEvent&);
 
-    void OnSetFocus(wxFocusEvent &e) { if (m_Gfx) m_Gfx->SetFocus(); }
+    void OnSetFocus(wxFocusEvent &) { if (m_Gfx) m_Gfx->SetFocus(); }
+
+    void OnPrintUpdate(wxUpdateUIEvent &ui) { ui.Enable(!m_File.empty()); }
 
     // temporary bodges until event handling problem is sorted out:
     void OnDefaultsUpdate(wxUpdateUIEvent& event) { if (m_Gfx) m_Gfx->OnDefaultsUpdate(event); }
@@ -332,56 +303,48 @@ public:
     void OnViewClinoUpdate(wxUpdateUIEvent& event) { if (m_Gfx) m_Gfx->OnViewClinoUpdate(event); }
     void OnReverseDirectionOfRotationUpdate(wxUpdateUIEvent& event) { if (m_Gfx) m_Gfx->OnReverseDirectionOfRotationUpdate(event); }
     void OnCancelDistLineUpdate(wxUpdateUIEvent& event) { if (m_Gfx) m_Gfx->OnCancelDistLineUpdate(event); }
-#ifdef AVENGL
-    void OnAntiAliasUpdate(wxUpdateUIEvent& event) { if (m_Gfx) m_Gfx->OnAntiAliasUpdate(event); }
-    void OnSolidSurfaceUpdate(wxUpdateUIEvent& event) { if (m_Gfx) m_Gfx->OnSolidSurfaceUpdate(event); }
-#endif
     void OnIndicatorsUpdate(wxUpdateUIEvent& event) { if (m_Gfx) m_Gfx->OnIndicatorsUpdate(event); }
 
-    void OnDefaults(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnDefaults(); }
-    void OnPlan(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnPlan(); }
-    void OnElevation(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnElevation(); }
-    void OnDisplayOverlappingNames(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnDisplayOverlappingNames(); }
-    void OnShowCrosses(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShowCrosses(); }
-    void OnShowEntrances(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShowEntrances(); }
-    void OnShowFixedPts(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShowFixedPts(); }
-    void OnShowExportedPts(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShowExportedPts(); }
-    void OnShowStationNames(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShowStationNames(); }
-    void OnShowSurveyLegs(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShowSurveyLegs(); }
-    void OnShowSurface(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShowSurface(); }
-    void OnShowSurfaceDepth(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShowSurfaceDepth(); }
-    void OnShowSurfaceDashed(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShowSurfaceDashed(); }
-    void OnMoveEast(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnMoveEast(); }
-    void OnMoveNorth(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnMoveNorth(); }
-    void OnMoveSouth(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnMoveSouth(); }
-    void OnMoveWest(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnMoveWest(); }
-    void OnStartRotation(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnStartRotation(); }
-    void OnToggleRotation(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnToggleRotation(); }
-    void OnStopRotation(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnStopRotation(); }
-    void OnReverseControls(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnReverseControls(); }
-    void OnSlowDown(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnSlowDown(); }
-    void OnSpeedUp(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnSpeedUp(); }
-    void OnStepOnceAnticlockwise(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnStepOnceAnticlockwise(); }
-    void OnStepOnceClockwise(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnStepOnceClockwise(); }
-    void OnHigherViewpoint(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnHigherViewpoint(); }
-    void OnLowerViewpoint(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnLowerViewpoint(); }
-    void OnShiftDisplayDown(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShiftDisplayDown(); }
-    void OnShiftDisplayLeft(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShiftDisplayLeft(); }
-    void OnShiftDisplayRight(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShiftDisplayRight(); }
-    void OnShiftDisplayUp(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnShiftDisplayUp(); }
-    void OnZoomIn(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnZoomIn(); }
-    void OnZoomOut(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnZoomOut(); }
-    void OnToggleScalebar(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnToggleScalebar(); }
-    void OnToggleDepthbar(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnToggleDepthbar(); }
-    void OnViewCompass(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnViewCompass(); }
-    void OnViewClino(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnViewClino(); }
-    void OnViewGrid(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnViewGrid(); }
-    void OnReverseDirectionOfRotation(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnReverseDirectionOfRotation(); }
-    void OnCancelDistLine(wxCommandEvent& event) { if (m_Gfx) m_Gfx->OnCancelDistLine(); }
-#ifdef AVENGL
-    void OnAntiAlias(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnAntiAlias(); }
-    void OnSolidSurface(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnSolidSurface(); }
-#endif
+    void OnDefaults(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnDefaults(); }
+    void OnPlan(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnPlan(); }
+    void OnElevation(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnElevation(); }
+    void OnDisplayOverlappingNames(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnDisplayOverlappingNames(); }
+    void OnShowCrosses(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShowCrosses(); }
+    void OnShowEntrances(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShowEntrances(); }
+    void OnShowFixedPts(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShowFixedPts(); }
+    void OnShowExportedPts(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShowExportedPts(); }
+    void OnShowStationNames(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShowStationNames(); }
+    void OnShowSurveyLegs(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShowSurveyLegs(); }
+    void OnShowSurface(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShowSurface(); }
+    void OnShowSurfaceDepth(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShowSurfaceDepth(); }
+    void OnShowSurfaceDashed(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShowSurfaceDashed(); }
+    void OnMoveEast(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnMoveEast(); }
+    void OnMoveNorth(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnMoveNorth(); }
+    void OnMoveSouth(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnMoveSouth(); }
+    void OnMoveWest(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnMoveWest(); }
+    void OnStartRotation(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnStartRotation(); }
+    void OnToggleRotation(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnToggleRotation(); }
+    void OnStopRotation(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnStopRotation(); }
+    void OnReverseControls(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnReverseControls(); }
+    void OnSlowDown(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnSlowDown(); }
+    void OnSpeedUp(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnSpeedUp(); }
+    void OnStepOnceAnticlockwise(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnStepOnceAnticlockwise(); }
+    void OnStepOnceClockwise(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnStepOnceClockwise(); }
+    void OnHigherViewpoint(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnHigherViewpoint(); }
+    void OnLowerViewpoint(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnLowerViewpoint(); }
+    void OnShiftDisplayDown(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShiftDisplayDown(); }
+    void OnShiftDisplayLeft(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShiftDisplayLeft(); }
+    void OnShiftDisplayRight(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShiftDisplayRight(); }
+    void OnShiftDisplayUp(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnShiftDisplayUp(); }
+    void OnZoomIn(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnZoomIn(); }
+    void OnZoomOut(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnZoomOut(); }
+    void OnToggleScalebar(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnToggleScalebar(); }
+    void OnToggleDepthbar(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnToggleDepthbar(); }
+    void OnViewCompass(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnViewCompass(); }
+    void OnViewClino(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnViewClino(); }
+    void OnViewGrid(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnViewGrid(); }
+    void OnReverseDirectionOfRotation(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnReverseDirectionOfRotation(); }
+    void OnCancelDistLine(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnCancelDistLine(); }
     // end of horrible bodges
 
     void OnToggleMetric(wxCommandEvent&) { if (m_Gfx) m_Gfx->OnToggleMetric(); }
@@ -403,34 +366,6 @@ public:
     Double GetZMin() const { return m_ZMin; }
     Double GetZMax() const { return m_ZMin + m_ZExt; }
 
-#ifdef AVENGL
-    int GetTerrainXSize() const { return m_TerrainSize.x; }
-    int GetTerrainYSize() const { return m_TerrainSize.y; }
-
-    Double GetTerrainMinX() const { return m_TerrainExtents.xmin; }
-    Double GetTerrainMaxX() const { return m_TerrainExtents.xmax; }
-    Double GetTerrainMinY() const { return m_TerrainExtents.ymin; }
-    Double GetTerrainMaxY() const { return m_TerrainExtents.ymax; }
-    Double GetTerrainMinZ() const { return m_TerrainExtents.zmin; }
-    Double GetTerrainMaxZ() const { return m_TerrainExtents.zmax; }
-
-    Double GetTerrainXSquareSize() const {
-	return (m_TerrainExtents.xmax - m_TerrainExtents.xmin)
-		/ m_TerrainSize.x;
-    }
-    Double GetTerrainYSquareSize() const {
-	return (m_TerrainExtents.ymax - m_TerrainExtents.ymin)
-		/ m_TerrainSize.y;
-    }
-
-    Double GetTerrainHeight(int x, int y) const {
-	assert(x >= 0 && x < m_TerrainSize.x);
-	assert(y >= 0 && y < m_TerrainSize.y);
-
-	return m_TerrainGrid[x + m_TerrainSize.x * y];
-    }
-#endif
-
     int GetNumLegs() const { return m_NumLegs; }
     int GetNumPoints() const { return m_NumPoints; }
     int GetNumCrosses() const { return m_NumCrosses; }
@@ -483,22 +418,31 @@ public:
 	return m_Points[band].begin();
     }
 
-    list<LabelInfo*>::const_iterator GetLabels() const {
-	return m_Labels.begin();
-    }
-
     list<PointInfo*>::const_iterator GetPointsEnd(int band) const {
 	assert(band >= 0 && band < NUM_DEPTH_COLOURS);
 	return m_Points[band].end();
     }
 
+    list<LabelInfo*>::const_iterator GetLabels() const {
+	return m_Labels.begin();
+    }
+
     list<LabelInfo*>::const_iterator GetLabelsEnd() const {
 	return m_Labels.end();
     }
 
-    void ShowInfo(LabelInfo *label);
-    void DisplayTreeInfo(wxTreeItemData* data);
+    list<LabelInfo*>::const_reverse_iterator GetRevLabels() const {
+	return m_Labels.rbegin();
+    }
+
+    list<LabelInfo*>::const_reverse_iterator GetRevLabelsEnd() const {
+	return m_Labels.rend();
+    }
+
+    void ShowInfo(const LabelInfo *label);
+    void DisplayTreeInfo(const wxTreeItemData* data);
     void TreeItemSelected(wxTreeItemData* data);
+    wxPageSetupData * GetPageSetupData() { return &m_pageSetupData; }
 
 private:
     DECLARE_EVENT_TABLE()
