@@ -1,7 +1,7 @@
 /* netskel.c
  * Survex network reduction - remove trailing traverses and concatenate
  * traverses between junctions
- * Copyright (C) 1991-2001 Olly Betts
+ * Copyright (C) 1991-2002 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -112,8 +112,8 @@ solve_network(void /*node *stnlist*/)
        * first station added is the last in the list. */
       FOR_EACH_STN(stn, stnlist) stnFirst = stn;
 
-      ASSERT2(stnFirst->leg[0], "no fixed stns, but we've got a zero node!");
-      ASSERT2(stnFirst, "no stations left in net!");
+      SVX_ASSERT2(stnFirst->leg[0], "no fixed stns, but we've got a zero node!");
+      SVX_ASSERT2(stnFirst, "no stations left in net!");
       stn = stnFirst;
       printf(msg(/*Survey has no fixed points. Therefore I've fixed %s at (0,0,0)*/72),
 	     sprint_prefix(stn->name));
@@ -237,7 +237,7 @@ concatenate_trav(node *stn, int i)
    trav->join1 = stn->leg[i];
 
    j = reverse_leg_dirn(stn->leg[i]);
-   ASSERT(j == 0 || j == 1);
+   SVX_ASSERT(j == 0 || j == 1);
 
    newleg = copy_link(stn->leg[i]);
 
@@ -275,7 +275,7 @@ concatenate_trav(node *stn, int i)
 
 #if PRINT_NETBITS
    putchar(' ');
-   print_var(newleg->v);
+   print_var(&(newleg->v));
    printf("\nStacked ");
    print_prefix(newleg2->l.to->name);
    printf(",%d-", reverse_leg_dirn(newleg2));
@@ -327,7 +327,7 @@ printf( " v = ( %.2f, %.2f, %.2f )\n", v[0], v[1], v[2] );
    s = sqrd(cx) + sqrd(cy);
    if (s > 0.0) {
       s = hsqrd / s;
-      ASSERT(s >= 0.0);
+      SVX_ASSERT(s >= 0.0);
       s = sqrt(s);
       s = 1 - s;
       tot = sqrd(cx * s) + sqrd(cy * s) + sqrd(e[2]);
@@ -357,7 +357,7 @@ printf( " v = ( %.2f, %.2f, %.2f )\n", v[0], v[1], v[2] );
       s = sqrd(nx) + sqrd(ny) + sqrd(cz);
       if (s > 0.0) {
 	 s = rsqrd / s;
-	 ASSERT(s >= 0);
+	 SVX_ASSERT(s >= 0);
 	 s = sqrt(s);
 	 tot = sqrd(cx - s * nx) + sqrd(cy - s * ny) + sqrd(cz - s * cz);
 	 if (tot <= expected_error) {
@@ -391,7 +391,6 @@ replace_travs(void)
    delta e, sc;
    bool fEquate; /* used to indicate equates in output */
    int cLegsTrav = 0;
-   prefix *nmPrev = NULL;
    bool fArtic;
 
    out_current_action(msg(/*Calculating traverses between nodes*/127));
@@ -401,18 +400,18 @@ replace_travs(void)
 
    if (!pimg) {
       char *fnm;
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       if (fUseNewFormat) {
 	 fnm = add_ext(fnm_output_base, EXT_SVX_3DX);
       } else {
 #endif
 	 fnm = add_ext(fnm_output_base, EXT_SVX_3D);
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       }
 #endif
       filename_register_output(fnm);
 
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       if (fUseNewFormat) {
 	 pimg = cave_open_write(fnm, survey_title);
 	 if (!pimg) fatalerror(cave_error(), fnm);
@@ -420,7 +419,7 @@ replace_travs(void)
 #endif
 	 pimg = img_open_write(fnm, survey_title, fTrue);
 	 if (!pimg) fatalerror(img_error(), fnm);
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       }
 #endif
       osfree(fnm);
@@ -428,12 +427,17 @@ replace_travs(void)
 
    /* First do all the one leg traverses */
    FOR_EACH_STN(stn1, stnlist) {
+#if PRINT_NETBITS
+      printf("One leg traverses from ");
+      print_prefix(stn1->name);
+      printf(" [%p]\n", stn1);
+#endif
       for (i = 0; i <= 2; i++) {
 	 linkfor *leg = stn1->leg[i];
 	 if (leg && data_here(leg) &&
 	     !(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
-	    ASSERT(fixed(stn1));
-	    ASSERT(!fZeros(&leg->v));
+	    SVX_ASSERT(fixed(stn1));
+	    SVX_ASSERT(!fZeros(&leg->v));
 
 	    stn2 = leg->l.to;
 	    if (TSTBIT(leg->l.flags, FLAGS_SURFACE)) {
@@ -443,7 +447,7 @@ replace_travs(void)
 	       stn1->name->sflags |= BIT(SFLAGS_UNDERGROUND);
 	       stn2->name->sflags |= BIT(SFLAGS_UNDERGROUND);
 	    }
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
 	    if (!fUseNewFormat) {
 #endif
 	       img_write_item(pimg, img_MOVE, 0, NULL,
@@ -451,7 +455,7 @@ replace_travs(void)
 	       img_write_item(pimg, img_LINE, leg->l.flags,
 			      sprint_prefix(stn1->name->up),
 			      POS(stn2, 0), POS(stn2, 1), POS(stn2, 2));
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
 	    }
 #endif
 	    if (!(leg->l.reverse & FLAG_ARTICULATION)) {
@@ -482,7 +486,7 @@ replace_travs(void)
 		  eTotTheo = hTotTheo + vTotTheo;
 #endif
 #ifdef BLUNDER_DETECTION
-		  memcpy(&err, &e, sizeof(d));
+		  memcpy(&err, &e, sizeof(delta));
 		  do_blunder = (eTot > eTotTheo);
 		  fputs("\ntraverse ", fhErrStat);
 		  fprint_prefix(fhErrStat, stn1->name);
@@ -517,18 +521,18 @@ replace_travs(void)
 #if PRINT_NETBITS
       printf(" Trav ");
       print_prefix(stn1->name);
-      printf("<%p>%s...%s", stn1, szLink, szLink);
+      printf("<%p>[%d]%s...%s", stn1, i, szLink, szLink);
       print_prefix(stn2->name);
-      printf("<%p>\n", stn2);
+      printf("<%p>[%d]\n", stn2, j);
 #endif
 
-      ASSERT(fixed(stn1));
-      ASSERT(fixed(stn2));
+      SVX_ASSERT(fixed(stn1));
+      SVX_ASSERT(fixed(stn2));
 
       /* calculate scaling factors for error distribution */
       eTot = 0.0;
       hTot = vTot = 0.0;
-      ASSERT(data_here(stn1->leg[i]));
+      SVX_ASSERT(data_here(stn1->leg[i]));
       if (fZeros(&stn1->leg[i]->v)) {
 	 sc[0] = sc[1] = sc[2] = 0.0;
       } else {
@@ -550,13 +554,12 @@ replace_travs(void)
       eTotTheo = hTotTheo + vTotTheo;
       cLegsTrav = 0;
       lenTrav = 0.0;
-      nmPrev = stn1->name;
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       if (!fUseNewFormat) {
 #endif
 	 img_write_item(pimg, img_MOVE, 0, NULL,
-			   POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
-#ifdef NEW3DFORMAT
+			POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
+#ifdef CHASM3DX
       }
 #endif
 
@@ -570,7 +573,7 @@ replace_travs(void)
 #ifdef BLUNDER_DETECTION
       delta err;
       int do_blunder;
-      memcpy(&err, &e, sizeof(d));
+      memcpy(&err, &e, sizeof(delta));
       do_blunder = (eTot > eTotTheo);
       if (fhErrStat && !fArtic) {
 	 fputs("\ntraverse ", fhErrStat);
@@ -591,7 +594,7 @@ replace_travs(void)
 	  * should have stn3->leg[k]->l.to == stn1 */
 	 stn3 = stn1->leg[i]->l.to;
 	 k = reverse_leg_dirn(stn1->leg[i]);
-	 ASSERT2(stn3->leg[k]->l.to == stn1,
+	 SVX_ASSERT2(stn3->leg[k]->l.to == stn1,
 		 "reverse leg doesn't reciprocate");
 
 	 reached_end = (stn3 == stn2 && k == j);
@@ -618,51 +621,50 @@ replace_travs(void)
 
 	 lenTot = sqrdd(leg->d);
 
+	 if (!fZeros(&leg->v)) fEquate = fFalse;
 	 if (!reached_end) {
 	    add_stn_to_list(&stnlist, stn3);
-	    if (!fZeros(&leg->v)) {
-	       fEquate = fFalse;
+	    if (!fEquate) {
 	       mulsd(&e, &leg->v, &sc);
 	       adddd(&POSD(stn3), &POSD(stn3), &e);
 	    }
 	    fix(stn3);
-	 } else {
-	    if (!fZeros(&leg->v)) fEquate = fFalse;
 	 }
 
-	 if (TSTBIT(leg->l.flags, FLAGS_SURFACE)) {
-	    stn1->name->sflags |= BIT(SFLAGS_SURFACE);
-	    stn3->name->sflags |= BIT(SFLAGS_SURFACE);
-	 } else {
-	    stn1->name->sflags |= BIT(SFLAGS_UNDERGROUND);
-	    stn3->name->sflags |= BIT(SFLAGS_UNDERGROUND);
-	 }
-#ifdef NEW3DFORMAT
-	 if (!fUseNewFormat) {
+	 if (!(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
+	     if (TSTBIT(leg->l.flags, FLAGS_SURFACE)) {
+		stn1->name->sflags |= BIT(SFLAGS_SURFACE);
+		stn3->name->sflags |= BIT(SFLAGS_SURFACE);
+	     } else {
+		stn1->name->sflags |= BIT(SFLAGS_UNDERGROUND);
+		stn3->name->sflags |= BIT(SFLAGS_UNDERGROUND);
+	     }
+
+#ifdef CHASM3DX
+	    if (!fUseNewFormat) {
 #endif
-	    if (!(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
-	       ASSERT(!fEquate);
-	       ASSERT(!fZeros(&leg->v));
+	       SVX_ASSERT(!fEquate);
+	       SVX_ASSERT(!fZeros(&leg->v));
 	       img_write_item(pimg, img_LINE, leg->l.flags,
 			      sprint_prefix(leg_pfx),
 			      POS(stn3, 0), POS(stn3, 1), POS(stn3, 2));
+#ifdef CHASM3DX
 	    }
-#ifdef NEW3DFORMAT
-	 }
 #endif
+	 }
 
 	 /* FIXME: equate at the start of a traverse treated specially
 	  * - what about equates at end? */
-	 if (nmPrev != stn3->name && !(fEquate && cLegsTrav == 0)) {
-	    /* (node not part of same stn) &&
+	 if (stn1->name != stn3->name && !(fEquate && cLegsTrav == 0)) {
+ 	    /* (node not part of same stn) &&
 	     * (not equate at start of traverse) */
 #ifndef BLUNDER_DETECTION
 	    if (fhErrStat && !fArtic) {
-	       if (!nmPrev->ident) {
+	       if (!stn1->name->ident) {
 		  /* FIXME: not ideal */
 		  fputs("<fixed point>", fhErrStat);
 	       } else {
-		  fprint_prefix(fhErrStat, nmPrev);
+		  fprint_prefix(fhErrStat, stn1->name);
 	       }
 	       fputs(fEquate ? szLinkEq : szLink, fhErrStat);
 	       if (reached_end) {
@@ -675,18 +677,9 @@ replace_travs(void)
 	       }
 	    }
 #endif
-	    nmPrev = stn3->name;
 	    if (!fEquate) {
 	       cLegsTrav++;
 	       lenTrav += sqrt(lenTot);
-	    } else if (lenTot > 0.0) {
-#if DEBUG_INVALID
-	       fprintf(stderr, "lenTot = %8.4f ", lenTot);
-	       fprint_prefix(stderr, nmPrev);
-	       fprintf(stderr, " -> ");
-	       fprint_prefix(stderr, stn3->name);
-#endif
-	       BUG("during calculation of closure errors");
 	    }
 	 } else {
 #if SHOW_INTERNAL_LEGS
@@ -695,7 +688,7 @@ replace_travs(void)
 	    if (lenTot > 0.0) {
 #if DEBUG_INVALID
 	       fprintf(stderr, "lenTot = %8.4f ", lenTot);
-	       fprint_prefix(stderr, nmPrev);
+	       fprint_prefix(stderr, stn1->name);
 	       fprintf(stderr, " -> ");
 	       fprint_prefix(stderr, stn3->name);
 #endif
@@ -779,13 +772,13 @@ replace_trailing_travs(void)
 	 stn1->leg[j] = stn1->leg[i];
       }
       stn1->leg[i] = ptrTrail->join1;
-      ASSERT(fixed(stn1));
-#ifdef NEW3DFORMAT
+      SVX_ASSERT(fixed(stn1));
+#ifdef CHASM3DX
       if (!fUseNewFormat) {
 #endif
 	 img_write_item(pimg, img_MOVE, 0, NULL,
 			POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       }
 #endif
 
@@ -813,23 +806,25 @@ replace_trailing_travs(void)
 
 	 fix(stn2);
 	 add_stn_to_list(&stnlist, stn2);
-	 if (TSTBIT(leg->l.flags, FLAGS_SURFACE)) {
-	    stn1->name->sflags |= BIT(SFLAGS_SURFACE);
-	    stn2->name->sflags |= BIT(SFLAGS_SURFACE);
-	 } else {
-	    stn1->name->sflags |= BIT(SFLAGS_UNDERGROUND);
-	    stn2->name->sflags |= BIT(SFLAGS_UNDERGROUND);
+	 if (!(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
+	     if (TSTBIT(leg->l.flags, FLAGS_SURFACE)) {
+		stn1->name->sflags |= BIT(SFLAGS_SURFACE);
+		stn2->name->sflags |= BIT(SFLAGS_SURFACE);
+	     } else {
+		stn1->name->sflags |= BIT(SFLAGS_UNDERGROUND);
+		stn2->name->sflags |= BIT(SFLAGS_UNDERGROUND);
+	     }
 	 }
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
 	 if (!fUseNewFormat) {
 #endif
 	    if (!(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
-	       ASSERT(!fZeros(&leg->v));
+	       SVX_ASSERT(!fZeros(&leg->v));
 	       img_write_item(pimg, img_LINE, leg->l.flags,
 			      sprint_prefix(leg_pfx),
 			      POS(stn2, 0), POS(stn2, 1), POS(stn2, 2));
 	    }
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
 	 }
 #endif
 
@@ -846,13 +841,13 @@ replace_trailing_travs(void)
    }
 
    /* write out connections with no survey data */
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
    if (!fUseNewFormat) {
 #endif
       while (nosurveyhead) {
 	 nosurveylink *p = nosurveyhead;
-	 ASSERT(fixed(p->fr));
-	 ASSERT(fixed(p->to));
+	 SVX_ASSERT(fixed(p->fr));
+	 SVX_ASSERT(fixed(p->to));
 	 if (TSTBIT(p->flags, FLAGS_SURFACE)) {
 	    p->fr->name->sflags |= BIT(SFLAGS_SURFACE);
 	    p->to->name->sflags |= BIT(SFLAGS_SURFACE);
@@ -868,16 +863,16 @@ replace_trailing_travs(void)
 	 nosurveyhead = p->next;
 	 osfree(p);
       }
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
    }
 #endif
 
    /* write stations to .3d file and free legs and stations */
    FOR_EACH_STN(stn1, stnlist) {
       int d;
-      ASSERT(fixed(stn1));
+      SVX_ASSERT(fixed(stn1));
       /* take care of unused fixed points */
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       if (!fUseNewFormat) {
 #endif
 	 if (stn1->name->stn == stn1 && stn1->name->ident) {
@@ -891,12 +886,13 @@ replace_trailing_travs(void)
 			      POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
 	    }
 	 }
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       }
 #endif
       /* update coords of bounding box, ignoring the base positions
-       * of points fixed with error estimates */
-      if (stn1->name->ident) {
+       * of points fixed with error estimates and only counting stations
+       * in underground surveys. */
+      if (stn1->name->ident && TSTBIT(stn1->name->sflags, SFLAGS_UNDERGROUND)) {
 	 for (d = 0; d < 3; d++) {
 	    if (POS(stn1, d) < min[d]) {
 	       min[d] = POS(stn1, d);
@@ -939,8 +935,8 @@ replace_trailing_travs(void)
 	    stnB = leg->l.to;
 	    iB = reverse_leg_dirn(leg);
 	    legRev = stnB->leg[iB];
-	    ASSERT2(legRev->l.to == stn1, "leg doesn't reciprocate");
-	    ASSERT(fixed(stn1));
+	    SVX_ASSERT2(legRev->l.to == stn1, "leg doesn't reciprocate");
+	    SVX_ASSERT(fixed(stn1));
 	    if (!(leg->l.flags &
 		  (BIT(FLAGS_DUPLICATE)|BIT(FLAGS_SPLAY)|
 		   BIT(FLAGS_SURFACE)))) {
@@ -950,7 +946,7 @@ replace_trailing_travs(void)
 				 sqrd(POS(stnB, 1) - POS(stn1, 1)) +
 				 sqrd(POS(stnB, 2) - POS(stn1, 2)));
 		  total += sqrt(sqrdd(leg->d));
-		  totplan += radius(leg->d[0], leg->d[1]);
+		  totplan += hypot(leg->d[0], leg->d[1]);
 		  totvert += fabs(leg->d[2]);
 	       }
 	    }
