@@ -67,7 +67,7 @@ static bool fShots = fTrue;
 static bool fSurface = fFalse;
 static bool fSkipBlank = fFalse;
 
-bool fBorder = fTrue, fCutlines = fTrue;
+bool fBorder = fTrue, fCutlines = fTrue, fRaw = fFalse;
 bool fBlankPage = fFalse;
 
 static img *pimg;
@@ -108,17 +108,20 @@ pick_scale(int x, int y)
 #if 0
    double E;
 #endif
-   /*    pagesY = ceil((image_dy+61.0)/PaperDepth)
-    * so (image_dy+61.0)/PaperDepth <= pagesY < (image_dy+61.0)/PaperDepth+1
-    * so image_dy <= pagesY*PaperDepth-61 < image_dy+PaperDepth
+   /*    pagesY = ceil((image_dy+allow)/PaperDepth)
+    * so (image_dy+allow)/PaperDepth <= pagesY < (image_dy+allow)/PaperDepth+1
+    * so image_dy <= pagesY*PaperDepth-allow < image_dy+PaperDepth
     * and Sc = image_dy / (yMax-yMin)
-    * so Sc <= (pagesY*PaperDepth-61)/(yMax-yMin) < Sc+PaperDepth/(yMax-yMin)
+    * so Sc <= (pagesY*PaperDepth-allow)/(yMax-yMin) < Sc+PaperDepth/(yMax-yMin)
     */
    Sc_x = Sc_y = DEF_RATIO;
    if (PaperWidth > 0.0 && xMax > xMin)
       Sc_x = (x * PaperWidth - 19.0) / (xMax - xMin);
-   if (PaperDepth > 0.0 && yMax > yMin)
-      Sc_y = (y * PaperDepth - 61.0) / (yMax - yMin);
+   if (PaperDepth > 0.0 && yMax > yMin) {
+      double allow = 21.0;
+      if (!fRaw) allow += (view == EXTELEV ? 30.0 : 40.0);
+      Sc_y = (y * PaperDepth - allow) / (yMax - yMin);
+   }
 
    Sc_x = min(Sc_x, Sc_y) * 0.99; /* shrink by 1% so we don't cock up */
 #if 0 /* this picks a nice (in some sense) ratio, but is too stingy */
@@ -135,6 +138,9 @@ pages_required(double Sc)
    double image_centre_x, image_centre_y;
    double paper_centre_x, paper_centre_y;
 
+   double allow = 21.0;
+   if (!fRaw) allow += (view == EXTELEV ? 30.0 : 40.0);
+
    image_dx = (xMax - xMin) * Sc;
    if (PaperWidth > 0.0) {
       pagesX = (int)ceil((image_dx + 19.0) / PaperWidth);
@@ -149,11 +155,11 @@ pages_required(double Sc)
 
    image_dy = (yMax - yMin) * Sc;
    if (PaperDepth > 0.0) {
-      pagesY = (int)ceil((image_dy + 61.0) / PaperDepth);
+      pagesY = (int)ceil((image_dy + allow) / PaperDepth);
    } else {
       /* paperdepth not fixed (eg window or roll printer/plotter) */
       pagesY = 1;
-      PaperDepth = image_dy + 61.0;
+      PaperDepth = image_dy + allow;
    }
    paper_centre_y = 20 + (pagesY * PaperDepth) / 2;
    image_centre_y = Sc * (yMax + yMin) / 2;
@@ -167,6 +173,8 @@ draw_info_box(double num, double denom)
    int boxwidth = 60;
    int boxheight = 30;
 
+   if (pr->SetColour) pr->SetColour(PR_COLOUR_FRAME);
+
    if (view != EXTELEV) {
       boxwidth = 100;
       boxheight = 40;
@@ -175,11 +183,13 @@ draw_info_box(double num, double denom)
       MOVEMM(0, 30); DRAWMM(60, 30);
    }
 
-   MOVEMM(0, 0);
-   DRAWMM(0, boxheight);
+   MOVEMM(0, boxheight);
    DRAWMM(boxwidth, boxheight);
    DRAWMM(boxwidth, 0);
-   DRAWMM(0, 0);
+   if (!fBorder) {
+      DRAWMM(0, 0);
+      DRAWMM(0, boxheight);
+   }
 
    MOVEMM(0, 20); DRAWMM(60, 20);
    MOVEMM(0, 10); DRAWMM(60, 10);
@@ -189,13 +199,10 @@ draw_info_box(double num, double denom)
       long ax, ay, bx, by, cx, cy, dx, dy;
       int c;
 
-      MOVEMM(62, 36);
-      pr->WriteString(msg(/*North*/115));
-
       ax = (long)((80 - 15 * sin(rad(000.0 + rot))) * scX);
       ay = (long)((20 + 15 * cos(rad(000.0 + rot))) * scY);
-      bx = (long)((80 - 07 * sin(rad(180.0 + rot))) * scX);
-      by = (long)((20 + 07 * cos(rad(180.0 + rot))) * scY);
+      bx = (long)((80 -  7 * sin(rad(180.0 + rot))) * scX);
+      by = (long)((20 +  7 * cos(rad(180.0 + rot))) * scY);
       cx = (long)((80 - 15 * sin(rad(160.0 + rot))) * scX);
       cy = (long)((20 + 15 * cos(rad(160.0 + rot))) * scY);
       dx = (long)((80 - 15 * sin(rad(200.0 + rot))) * scX);
@@ -219,14 +226,15 @@ draw_info_box(double num, double denom)
 	    DRAWMM(80.0 + RADIUS * sin(rad(c)), 20.0 + RADIUS * cos(rad(c)));
       }
 
+      if (pr->SetColour) pr->SetColour(PR_COLOUR_TEXT);
+      MOVEMM(62, 36);
+      pr->WriteString(msg(/*North*/115));
+
       MOVEMM(5, 23);
       pr->WriteString(msg(/*Plan view*/117));
       break;
     }
     case ELEV: case TILT:
-      MOVEMM(62, 33);
-      pr->WriteString(msg(/*Elevation on*/116));
-
       MOVEMM(65, 15); DRAWMM(70, 12); DRAWMM(68, 15); DRAWMM(70, 18);
 
       DRAWMM(65, 15); DRAWMM(95, 15);
@@ -235,6 +243,10 @@ draw_info_box(double num, double denom)
 
       MOVEMM(80, 13); DRAWMM(80, 17);
 
+      if (pr->SetColour) pr->SetColour(PR_COLOUR_TEXT);
+      MOVEMM(62, 33);
+      pr->WriteString(msg(/*Elevation on*/116));
+
       sprintf(szTmp, "%03d"DEG, (rot + 270) % 360);
       MOVEMM(65, 20); pr->WriteString(szTmp);
       sprintf(szTmp, "%03d"DEG, (rot + 90) % 360);
@@ -244,6 +256,7 @@ draw_info_box(double num, double denom)
       pr->WriteString(msg(/*Elevation*/118));
       break;
     case EXTELEV:
+      if (pr->SetColour) pr->SetColour(PR_COLOUR_TEXT);
       MOVEMM(5, 13);
       pr->WriteString(msg(/*Extended elevation*/191));
       break;
@@ -324,17 +337,20 @@ draw_scale_bar(double x, double y, double MaxLength, double scale)
    p = buf + strlen(buf);
    sprintf(p, " (%.0f%s)", (double)pow(10.0, (double)E), u_buf);
 
+   if (pr->SetColour) pr->SetColour(PR_COLOUR_TEXT);
    MOVEMM(x, y + 4); pr->WriteString(buf);
 
    /* Work out how many divisions there will be */
    n = (int)(MaxLength / d);
 
+   if (pr->SetColour) pr->SetColour(PR_COLOUR_FRAME);
    /* Draw top and bottom sides of scale bar */
    MOVEMM(x, y + 3); DRAWMM(x + n * d, y + 3);
    MOVEMM(x + n * d, y); DRAWMM(x, y);
 
    /* Draw divisions and label them */
    for (c = 0; c <= n; c++) {
+      if (pr->SetColour) pr->SetColour(PR_COLOUR_FRAME);
       MOVEMM(x + c * d, y); DRAWMM(x + c * d, y + 3);
 #if 0
       /* Gives a "zebra crossing" scale bar if we've a FillBox function */
@@ -345,6 +361,7 @@ draw_scale_bar(double x, double y, double MaxLength, double scale)
        * implementations return char* (ptr to end of written string I think) */
       sprintf(buf, "%d", c * Step);
       l = strlen(buf);
+      if (pr->SetColour) pr->SetColour(PR_COLOUR_TEXT);
       MOVEMM(x + c * d - l, y - 4);
       pr->WriteString(buf);
    }
@@ -356,8 +373,8 @@ getanswer(char *szReplies)
 {
    int ch;
    char *reply;
-   ASSERT2(szReplies, "NULL pointer passed");
-   ASSERT2(*szReplies, "list of possible replies is empty");
+   SVX_ASSERT2(szReplies, "NULL pointer passed");
+   SVX_ASSERT2(*szReplies, "list of possible replies is empty");
    putchar('(');
    putchar(*szReplies);
    for (reply = szReplies + 1; *reply; reply++) {
@@ -449,13 +466,20 @@ read_in_data(void)
        case img_BAD:
 	 return 0;
        case img_LINE:
-	 /* if we're plotting surface legs or this isn't a surface leg */
-	 if (fSurface || !(pimg->flags & img_FLAG_SURFACE)) {
+	 /* if we're plotting underground legs and this is one or
+	  *    we're plotting surface legs and this is one
+	  */
+	 if ((fShots && !(pimg->flags & img_FLAG_SURFACE)) ||
+	     (fSurface && (pimg->flags & img_FLAG_SURFACE))) {
 	    if (fMove) {
 	       fMove = fFalse;
 	       stack(img_MOVE, NULL, &p_move);
 	    }
-	    stack(img_LINE, NULL, &p);
+	    if (pimg->flags & img_FLAG_SURFACE) {
+	       stack(999, NULL, &p);
+	    } else {
+	       stack(img_LINE, NULL, &p);
+	    }
 	    break;
 	 }
 	 /* FALLTHRU */
@@ -464,8 +488,10 @@ read_in_data(void)
 	 fMove = fTrue;
 	 break;
        case img_LABEL:
-	 if (fSurface || (pimg->flags & img_SFLAG_UNDERGROUND))
-	    stack(img_LABEL, pimg->label, &p);
+	 if (fLabels || fCrosses) {
+	    if (fSurface || (pimg->flags & img_SFLAG_UNDERGROUND))
+	       stack(img_LABEL, pimg->label, &p);
+	 }
 	 break;
       }
    } while (result != img_STOP);
@@ -484,9 +510,9 @@ next_page(int *pstate, char **q, int pageLim)
    if (*pstate > 0) {
       /* doing a range */
       (*pstate)++;
-      ASSERT(*p == '-');
+      SVX_ASSERT(*p == '-');
       p++;
-      while (isspace(*p)) p++;
+      while (isspace((unsigned char)*p)) p++;
       if (sscanf(p, "%u%n", &page, &c) > 0) {
 	 p += c;
       } else {
@@ -499,7 +525,7 @@ next_page(int *pstate, char **q, int pageLim)
       return page;
    }
 
-   while (isspace(*p) || *p == ',') p++;
+   while (isspace((unsigned char)*p) || *p == ',') p++;
 
    if (!*p) return 0; /* done */
 
@@ -510,7 +536,7 @@ next_page(int *pstate, char **q, int pageLim)
    }
    if (sscanf(p, "%u%n", &page, &c) > 0) {
       p += c;
-      while (isspace(*p)) p++;
+      while (isspace((unsigned char)*p)) p++;
       *q = p;
       if (0 < page && page <= pageLim) {
 	 if (*p == '-') *pstate = page; /* range with start */
@@ -532,7 +558,7 @@ read_scale(const char *s)
 
    val = strtod(s, &p);
    if (val > 0 && p != s) {
-      while (isspace(*p)) p++;
+      while (isspace((unsigned char)*p)) p++;
       if (*p == '\0') {
 	 /* accept "<number>" as meaning "1:<number>" if number > 1
 	  * or "<number>:1" is number < 1 - so all these are the same scale:
@@ -554,7 +580,7 @@ read_scale(const char *s)
 	 optarg = p + 1;
 	 val2 = strtod(optarg, &p);
 	 if (val2 > 0 && p != optarg) {
-	    while (isspace(*p)) p++;
+	    while (isspace((unsigned char)*p)) p++;
 	    if (*p == '\0') {
 	       N_Scale = val;
 	       D_Scale = val2;
@@ -602,6 +628,7 @@ main(int argc, char **argv)
       {"crosses", no_argument, 0, 'c'},
       {"no-border", no_argument, 0, 'B'},
       {"no-cutlines", no_argument, 0, 'C'},
+      {"raw", no_argument, 0, 'r'},
       {"no-legs", no_argument, 0, 'l'},
       {"surface", no_argument, 0, 'S'},
       {"skip-blanks", no_argument, 0, 'k'},
@@ -612,7 +639,7 @@ main(int argc, char **argv)
       {0, 0, 0, 0}
    };
 
-#define short_opts "epb:t:s:ncBlSko:C"
+#define short_opts "epb:t:s:ncBlSko:Cr"
 
    /* TRANSLATE */
    static struct help_msg help[] = {
@@ -627,23 +654,24 @@ main(int argc, char **argv)
       {HLP_ENCODELONG(7),       "display crosses at stations"},
       {HLP_ENCODELONG(8),       "turn off page border"},
       {HLP_ENCODELONG(9),       "turn off dashed lines on internal page edges"},
-      {HLP_ENCODELONG(10),      "turn off display of survey legs"},
-      {HLP_ENCODELONG(11),      "turn on display of surface survey legs"},
-      {HLP_ENCODELONG(12),      "don't output blank pages"},
-      {HLP_ENCODELONG(13),      "set output file"},
-      {HLP_ENCODELONG(14),      "print out calibration page"},
+      {HLP_ENCODELONG(10),      "turn off infobox and page footer"},
+      {HLP_ENCODELONG(11),      "turn off display of survey legs"},
+      {HLP_ENCODELONG(12),      "turn on display of surface survey legs"},
+      {HLP_ENCODELONG(13),      "don't output blank pages"},
+      {HLP_ENCODELONG(14),      "set output file"},
+      {HLP_ENCODELONG(15),      "print out calibration page"},
       {0, 0}
    };
 
    msg_init(argv);
 
-   ASSERT(help[13].opt == HLP_ENCODELONG(13));
-   ASSERT(help[14].opt == HLP_ENCODELONG(14));
-   ASSERT(help[15].opt == 0);
-   ASSERT(!(pr->flags & PR_FLAG_CALIBRATE && pr->flags & PR_FLAG_NOFILEOUTPUT));
+   SVX_ASSERT(help[14].opt == HLP_ENCODELONG(14));
+   SVX_ASSERT(help[15].opt == HLP_ENCODELONG(15));
+   SVX_ASSERT(help[16].opt == 0);
+   SVX_ASSERT(!(pr->flags & PR_FLAG_CALIBRATE && pr->flags & PR_FLAG_NOFILEOUTPUT));
 
-   if (pr->flags & PR_FLAG_NOFILEOUTPUT) help[13] = help[15];
-   if (!(pr->flags & PR_FLAG_CALIBRATE)) help[14] = help[15];
+   if (pr->flags & PR_FLAG_NOFILEOUTPUT) help[14] = help[16];
+   if (!(pr->flags & PR_FLAG_CALIBRATE)) help[15] = help[16];
 
    fnm = NULL;
 
@@ -667,6 +695,9 @@ main(int argc, char **argv)
        case 'C': /* Cutlines */
 	 fCutlines = 0;
 	 break;
+       case 'r': /* Raw */
+	 fRaw = 1;
+	 break;
        case 'S': /* Surface */
 	 fSurface = 1;
 	 break;
@@ -694,7 +725,7 @@ main(int argc, char **argv)
 	 break;
        case 's':
 	 if (!read_scale(optarg)) {
-	    fatalerror(/*Bad scale `%s' (expecting e.g. `1:500', `500', or `0.002')*/217,
+	    fatalerror(/*Bad scale `%s' (expecting e.g. `1:500', `500', or `0.002')*/80,
 		       optarg);
 	 }
 	 fInteractive = fFalse;
@@ -776,8 +807,8 @@ main(int argc, char **argv)
 	 /* ini files searched in this order:
 	  * ~/.survex/print.ini [unix only]
 	  * /etc/survex/print.ini [unix only]
-	  * $SURVEXHOME/myprint.ini [not unix]
-	  * $SURVEXHOME/print.ini [must exist]
+	  * <support file directory>/myprint.ini [not unix]
+	  * <support file directory>/print.ini [must exist]
 	  */
 
 #if (OS==UNIX)
@@ -869,7 +900,7 @@ main(int argc, char **argv)
       if (fShots) *q++ = 'l';
       if (fSurface) *q++ = 's';
       *q = '\0';
-      printf(msg(/*Plot what (n = station names, c = crosses, l = legs, s = surface) (default &quot;%s&quot;)&#10;: */58), szTmp);
+      printf(msg(/*Plot what (n=station names, c=crosses, l=legs, s=surface) (default &quot;%s&quot;)&#10;: */58), szTmp);
       fgets(szTmp, sizeof(szTmp), stdin);
       if (szTmp[0] >= 32) {
 	  fLabels = (strchr(szTmp, 'n') != NULL);
@@ -960,7 +991,7 @@ main(int argc, char **argv)
       } else if (!read_scale(szTmp)) {
 	 size_t len = strlen(szTmp);
 	 if (len && szTmp[len - 1] == '\n') szTmp[len - 1] = '\0';
-	 printf(msg(/*Bad scale `%s' (expecting e.g. `1:500', `500', or `0.002')*/217),
+	 printf(msg(/*Bad scale `%s' (expecting e.g. `1:500', `500', or `0.002')*/80),
 		szTmp);
 	 fOk = fFalse;
 	 continue;
@@ -986,7 +1017,7 @@ main(int argc, char **argv)
 	 }
 
 	 p = szTmp;
-	 while (isspace(*p)) p++;
+	 while (isspace((unsigned char)*p)) p++;
 	 szPages = osstrdup(p);
 	 if (*szPages) {
 	    pages = page = state = 0;
@@ -996,7 +1027,7 @@ main(int argc, char **argv)
 	    while (1) {
 	       page = next_page(&state, &p, pageLim);
 	       if (state < 0) {
-		  printf(msg(/*Bad list of pages to print `%s'*/218), szPages);
+		  printf(msg(/*Bad list of pages to print `%s'*/179), szPages);
 		  putnl();
 		  fOk = fFalse;
 		  break;
@@ -1041,7 +1072,7 @@ main(int argc, char **argv)
       } else {
 	 page = next_page(&state, &p, pageLim);
       }
-      ASSERT(state >= 0); /* errors should have been caught above */
+      SVX_ASSERT(state >= 0); /* errors should have been caught above */
       if (page == 0) break;
       cPagesPrinted++;
       if (pages > 1) {
@@ -1060,13 +1091,14 @@ main(int argc, char **argv)
 	 li *pli;
 	 long x, y;
 	 int pending_move = 0;
+	 int last_leg_tag = 0;
 
 	 x = y = INT_MAX;
 
 	 if (pr->NewPage)
 	    pr->NewPage((int)page, pass, pagesX, pagesY);
 
-	 if ((int)page == (pagesY - 1) * pagesX + 1) {
+	 if (!fRaw && (int)page == (pagesY - 1) * pagesX + 1) {
 	    if (pr->SetFont) pr->SetFont(PR_FONT_DEFAULT);
 	    draw_info_box(N_Scale, D_Scale);
 	 }
@@ -1075,51 +1107,62 @@ main(int argc, char **argv)
 	 for (pli = pliHead; pli; pli = pli->pliNext) {
 	    if (pass == -1 && !fBlankPage) break;
 	    switch (pli->tag) {
-	     case img_MOVE:
-	       if (fShots) {
-		  long xnew, ynew;
-		  xnew = (long)((pli->to.x * Sc + xOrg) * scX);
-		  ynew = (long)((pli->to.y * Sc + yOrg) * scY);
+	     case img_MOVE: {
+	       long xnew, ynew;
+	       xnew = (long)((pli->to.x * Sc + xOrg) * scX);
+	       ynew = (long)((pli->to.y * Sc + yOrg) * scY);
 
-		  /* avoid superfluous moves */
-		  if (xnew != x || ynew != y) {
-		     x = xnew;
-		     y = ynew;
-		     pending_move = 1;
-		  }
+	       /* avoid superfluous moves */
+	       if (xnew != x || ynew != y) {
+		  x = xnew;
+		  y = ynew;
+		  pending_move = 1;
 	       }
 	       break;
-	     case img_LINE:
-	       if (fShots) {
-		  long xnew, ynew;
-		  xnew = (long)((pli->to.x * Sc + xOrg) * scX);
-		  ynew = (long)((pli->to.y * Sc + yOrg) * scY);
+	     }
+	     case img_LINE: case 999: {
+	       long xnew, ynew;
+	       xnew = (long)((pli->to.x * Sc + xOrg) * scX);
+	       ynew = (long)((pli->to.y * Sc + yOrg) * scY);
 
-		  if (pending_move) {
-		     pr->MoveTo(x, y);
-		     pending_move = 0;
-		  }
+	       if (pr->SetColour) {
+		  pr->SetColour(pli->tag == 999 ?
+				PR_COLOUR_SURFACE_LEG : PR_COLOUR_LEG);
+	       }
 
-		  /* avoid drawing superfluous lines */
-		  if (xnew != x || ynew != y) {
-		     x = xnew;
-		     y = ynew;
-		     pr->DrawTo(x, y);
-		  }
+	       if (pli->tag != last_leg_tag) pending_move = 1;
+
+	       if (pending_move) pr->MoveTo(x, y);
+
+	       /* avoid drawing superfluous lines */
+	       if (pending_move || xnew != x || ynew != y) {
+		  pending_move = 0;
+		  last_leg_tag = pli->tag;
+		  x = xnew;
+		  y = ynew;
+		  pr->DrawTo(x, y);
 	       }
 	       break;
-	     case img_LABEL:
-	       if (fCrosses || fLabels) {
-		  x = (long)((pli->to.x * Sc + xOrg) * scX);
-		  y = (long)((pli->to.y * Sc + yOrg) * scY);
+	     }
+	     case img_LABEL: {
+	       /* Only get here if (fCrosses || fLabels) - if neither
+		* is true, then img_LABEL doesn't get stacked */
+	       long xnew, ynew;
+	       xnew = (long)((pli->to.x * Sc + xOrg) * scX);
+	       ynew = (long)((pli->to.y * Sc + yOrg) * scY);
+	       if (fCrosses) {
+		  if (pr->SetColour) pr->SetColour(PR_COLOUR_CROSS);
+		  pr->DrawCross(xnew, ynew);
 	       }
-	       if (fCrosses) pr->DrawCross(x, y);
 	       if (fLabels) {
-		  pr->MoveTo(x, y);
+		  if (pr->SetColour) pr->SetColour(PR_COLOUR_LABELS);
+		  pr->MoveTo(xnew, ynew);
 		  pr->WriteString(pli->label);
 	       }
-	       if (fCrosses || fLabels) x = y = INT_MAX;
+	       /* Flag we need another MoveTo if (x,y) are to be reused. */
+	       pending_move = 1;
 	       break;
+	     }
 	    }
 	 }
 
@@ -1127,8 +1170,13 @@ main(int argc, char **argv)
 	    if (fBlankPage) break;
 	 } else {
 	    if (pr->ShowPage) {
-	       sprintf(szTmp, msg167, title, (int)page, pagesX * pagesY,
-		       datestamp);
+	       if (fRaw) {
+		  *szTmp = '\0';
+	       } else {
+		  sprintf(szTmp, msg167, title, (int)page, pagesX * pagesY,
+			  datestamp);
+	       }
+	       if (pr->SetColour) pr->SetColour(PR_COLOUR_TEXT);
 	       pr->ShowPage(szTmp);
 	    }
 	 }
@@ -1154,10 +1202,18 @@ drawticks(border clip, int tsize, int x, int y)
    int s = tsize * 4;
    int o = s / 8;
    bool fAtCorner = fFalse;
-   
-   if (x) {
-      pr->DrawTo(clip.x_min, clip.y_min + tsize);
-      if (fCutlines) {
+   if (pr->SetColour) pr->SetColour(PR_COLOUR_FRAME);
+   if (x == 0 && fBorder) {
+      /* solid left border */
+      pr->MoveTo(clip.x_min, clip.y_min);
+      pr->DrawTo(clip.x_min, clip.y_max);
+      fAtCorner = fTrue;
+   } else {
+      if (x > 0 || y > 0) {
+	 pr->MoveTo(clip.x_min, clip.y_min);
+	 pr->DrawTo(clip.x_min, clip.y_min + tsize);
+      }
+      if (x > 0 && fCutlines) {
 	 /* dashed left border */
 	 i = (clip.y_max - clip.y_min) -
 	     (tsize + ((clip.y_max - clip.y_min - tsize * 2L) % s) / 2);
@@ -1166,52 +1222,52 @@ drawticks(border clip, int tsize, int x, int y)
 	    pr->DrawTo(clip.x_min, clip.y_max - (i - o));
 	 }
       }
-      pr->MoveTo(clip.x_min, clip.y_max - tsize);
-      pr->DrawTo(clip.x_min, clip.y_max);
-      fAtCorner = fTrue;
-   } else if (fBorder) {
-      /* solid left border */
-      pr->MoveTo(clip.x_min, clip.y_min);
-      pr->DrawTo(clip.x_min, clip.y_max);
-      fAtCorner = fTrue;
-   } else if (x < pagesX - 1) {
-      pr->MoveTo(clip.x_min, clip.y_max - tsize);
-      pr->DrawTo(clip.x_min, clip.y_max);
-      fAtCorner = fTrue;
+      if (x > 0 || y < pagesY - 1) {
+	 pr->MoveTo(clip.x_min, clip.y_max - tsize);
+	 pr->DrawTo(clip.x_min, clip.y_max);
+	 fAtCorner = fTrue;
+      }
    }
 
-   if (y < pagesY - 1) {
+   if (y == pagesY - 1 && fBorder) {
+      /* solid top border */
       if (!fAtCorner) pr->MoveTo(clip.x_min, clip.y_max);
-      pr->DrawTo(clip.x_min + tsize, clip.y_max);
-      if (fCutlines) {
+      pr->DrawTo(clip.x_max, clip.y_max);
+      fAtCorner = fTrue;
+   } else {
+      if (y < pagesY - 1 || x > 0) {
+	 if (!fAtCorner) pr->MoveTo(clip.x_min, clip.y_max);
+	 pr->DrawTo(clip.x_min + tsize, clip.y_max);
+      }
+      if (y < pagesY - 1 && fCutlines) {
 	 /* dashed top border */
 	 i = (clip.x_max - clip.x_min) -
 	     (tsize + ((clip.x_max - clip.x_min - tsize * 2L) % s) / 2);
 	 for ( ; i > tsize; i -= s) {
 	    pr->MoveTo(clip.x_max - (i + o), clip.y_max);
 	    pr->DrawTo(clip.x_max - (i - o), clip.y_max);
-	    }
+	 }
       }
-      pr->MoveTo(clip.x_max - tsize, clip.y_max);
-      pr->DrawTo(clip.x_max, clip.y_max);
-      fAtCorner = fTrue;
-   } else if (fBorder) {
-      /* solid top border */
-      if (!fAtCorner) pr->MoveTo(clip.x_min, clip.y_max);
-      pr->DrawTo(clip.x_max, clip.y_max);
-      fAtCorner = fTrue;
-   } else if (y) {
-      pr->MoveTo(clip.x_max - tsize, clip.y_max);
-      pr->DrawTo(clip.x_max, clip.y_max);
+      if (y < pagesY - 1 || x < pagesX - 1) {
+	 pr->MoveTo(clip.x_max - tsize, clip.y_max);
+	 pr->DrawTo(clip.x_max, clip.y_max);
+	 fAtCorner = fTrue;
+      } else {
+	 fAtCorner = fFalse;
+      }
+   }
+
+   if (x == pagesX - 1 && fBorder) {
+      /* solid right border */
+      if (!fAtCorner) pr->MoveTo(clip.x_max, clip.y_max);
+      pr->DrawTo(clip.x_max, clip.y_min);
       fAtCorner = fTrue;
    } else {
-      fAtCorner = fFalse;
-   }
-      
-   if (x < pagesX - 1) {
-      if (!fAtCorner) pr->MoveTo(clip.x_max, clip.y_max);
-      pr->DrawTo(clip.x_max, clip.y_max - tsize);
-      if (fCutlines) {
+      if (x < pagesX - 1 || y < pagesY - 1) {
+	 if (!fAtCorner) pr->MoveTo(clip.x_max, clip.y_max);
+	 pr->DrawTo(clip.x_max, clip.y_max - tsize);
+      }
+      if (x < pagesX - 1 && fCutlines) {
 	 /* dashed right border */
 	 i = (clip.y_max - clip.y_min) -
 	     (tsize + ((clip.y_max - clip.y_min - tsize * 2L) % s) / 2);
@@ -1220,26 +1276,25 @@ drawticks(border clip, int tsize, int x, int y)
 	    pr->DrawTo(clip.x_max, clip.y_min + (i - o));
 	 }
       }
-      pr->MoveTo(clip.x_max, clip.y_min + tsize);
-      pr->DrawTo(clip.x_max, clip.y_min);
-      fAtCorner = fTrue;
-   } else if (fBorder) {
-      /* solid right border */
-      if (!fAtCorner) pr->MoveTo(clip.x_max, clip.y_max);
-      pr->DrawTo(clip.x_max, clip.y_min);
-      fAtCorner = fTrue;
-   } else if (x) {
-      pr->MoveTo(clip.x_max, clip.y_min + tsize);
-      pr->DrawTo(clip.x_max, clip.y_min);
-      fAtCorner = fTrue;
-   } else {
-      fAtCorner = fFalse;
+      if (x < pagesX - 1 || y > 0) {
+	 pr->MoveTo(clip.x_max, clip.y_min + tsize);
+	 pr->DrawTo(clip.x_max, clip.y_min);
+	 fAtCorner = fTrue;
+      } else {
+	 fAtCorner = fFalse;
+      }
    }
 
-   if (y) {
+   if (y == 0 && fBorder) {
+      /* solid bottom border */
       if (!fAtCorner) pr->MoveTo(clip.x_max, clip.y_min);
-      pr->DrawTo(clip.x_max - tsize, clip.y_min);
-      if (fCutlines) {
+      pr->DrawTo(clip.x_min, clip.y_min);
+   } else {
+      if (y > 0 || x < pagesX - 1) {
+	 if (!fAtCorner) pr->MoveTo(clip.x_max, clip.y_min);
+	 pr->DrawTo(clip.x_max - tsize, clip.y_min);
+      }
+      if (y > 0 && fCutlines) {
 	 /* dashed bottom border */
 	 i = (clip.x_max - clip.x_min) -
 	     (tsize + ((clip.x_max - clip.x_min - tsize * 2L) % s) / 2);
@@ -1248,15 +1303,10 @@ drawticks(border clip, int tsize, int x, int y)
 	    pr->DrawTo(clip.x_min + (i - o), clip.y_min);
 	 }
       }
-      pr->MoveTo(clip.x_min + tsize, clip.y_min);
-      pr->DrawTo(clip.x_min, clip.y_min);
-   } else if (fBorder) {
-      /* solid bottom border */
-      if (!fAtCorner) pr->MoveTo(clip.x_max, clip.y_min);
-      pr->DrawTo(clip.x_min, clip.y_min);
-   } else if (y < pagesY - 1) {
-      pr->MoveTo(clip.x_min + tsize, clip.y_min);
-      pr->DrawTo(clip.x_min, clip.y_min);
+      if (y > 0 || x > 0) {
+	 pr->MoveTo(clip.x_min + tsize, clip.y_min);
+	 pr->DrawTo(clip.x_min, clip.y_min);
+      }
    }
 }
 
@@ -1291,6 +1341,80 @@ as_int(const char *v, char *p, int min_val, int max_val)
    return (int)val;
 }
 
+/* Converts '0'-'9' to 0-9, 'A'-'F' to 10-15 and 'a'-'f' to 10-15.
+ * Undefined on other values */
+#define CHAR2HEX(C) (((C)+((C)>64?9:0))&15)
+
+unsigned long
+as_colour(const char *v, char *p)
+{
+   unsigned long val = 0xffffffff;
+   if (!p) setting_missing(v);
+   switch (tolower(*p)) {
+      case '#': {
+	 char *q = p + 1;
+	 while (isxdigit((unsigned char)*q)) q++;
+	 if (q - p == 4) {
+	    val = CHAR2HEX(p[1]) * 0x110000;
+	    val |= CHAR2HEX(p[2]) * 0x1100;
+	    val |= CHAR2HEX(p[3]) * 0x11;
+	 } else if (q - p == 7) {
+	    val = ((CHAR2HEX(p[1]) << 4) | CHAR2HEX(p[2])) << 16;
+	    val |= ((CHAR2HEX(p[3]) << 4) | CHAR2HEX(p[4])) << 8;
+	    val |= (CHAR2HEX(p[5]) << 4) | CHAR2HEX(p[6]);
+	 }
+	 break;
+      }
+      case 'a':
+	 if (strcasecmp(p, "aqua") == 0) val = 0x00fffful;
+	 break;
+      case 'b':
+	 if (strcasecmp(p, "black") == 0) val = 0x000000ul;
+	 else if (strcasecmp(p, "blue") == 0) val = 0x0000fful;
+	 break;
+      case 'f':
+	 if (strcasecmp(p, "fuchsia") == 0) val = 0xff00fful;
+	 break;
+      case 'g':
+	 if (strcasecmp(p, "gray") == 0) val = 0x808080ul;
+	 else if (strcasecmp(p, "green") == 0) val = 0x008000ul;
+	 break;
+      case 'l':
+	 if (strcasecmp(p, "lime") == 0) val = 0x00ff00ul;
+	 break;
+      case 'm':
+	 if (strcasecmp(p, "maroon") == 0) val = 0x800000ul;
+	 break;
+      case 'n':
+	 if (strcasecmp(p, "navy") == 0) val = 0x000080ul;
+	 break;
+      case 'o':
+	 if (strcasecmp(p, "olive") == 0) val = 0x808000ul;
+	 break;
+      case 'p':
+	 if (strcasecmp(p, "purple") == 0) val = 0x800080ul;
+	 break;
+      case 'r':
+	 if (strcasecmp(p, "red") == 0) val = 0xff0000ul;
+	 break;
+      case 's':
+	 if (strcasecmp(p, "silver") == 0) val = 0xc0c0c0ul;
+	 break;
+      case 't':
+	 if (strcasecmp(p, "teal") == 0) val = 0x008080ul;
+	 break;
+      case 'w':
+	 if (strcasecmp(p, "white") == 0) val = 0xfffffful;
+	 break;
+      case 'y':
+	 if (strcasecmp(p, "yellow") == 0) val = 0xffff00ul;
+	 break;
+   }
+   if (val == 0xffffffff) setting_bad_value(v, p);
+   osfree(p);
+   return val;
+}
+
 int
 as_bool(const char *v, char *p)
 {
@@ -1310,10 +1434,6 @@ as_double(const char *v, char *p, double min_val, double max_val)
    return val;
 }
 
-/* Converts '0'-'9' to 0-9, 'A'-'F' to 10-15 and 'a'-'f' to 10-15.
- * Undefined on other values */
-#define CHAR2HEX(C) (((C)+((C)>64?9:0))&15)
-
 /*
 Codes:
 \\ -> '\'
@@ -1346,7 +1466,8 @@ as_escstring(const char *v, char *s)
 	     case '\\': /* literal "\" */
 	       break;
 	     case 'x': /* hex digits */
-	       if (isxdigit(*p) && isxdigit(p[1])) {
+	       if (isxdigit((unsigned char)*p) &&
+		   isxdigit((unsigned char)p[1])) {
 		  if (pass) c = (CHAR2HEX(*p) << 4) | CHAR2HEX(p[1]);
 		  p += 2;
 		  break;
@@ -1376,7 +1497,7 @@ as_escstring(const char *v, char *s)
 	 if (pass) *q++ = c;
       }
       if (fSyntax) {
-	 ASSERT(pass == 0);
+	 SVX_ASSERT(pass == 0);
 	 setting_bad_value(v, s);
       }
    }
