@@ -1,6 +1,6 @@
 /* useful.h
  * Lots of oddments that come in handy generally
- * Copyright (C) 1993-2002 Olly Betts
+ * Copyright (C) 1993-2003 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #ifndef USEFUL_H
 #define USEFUL_H
 
+#include <config.h>
+
 #include <stdlib.h> /* for Borland C which #defines max() & min() there */
 #include <stdio.h>
 #include <math.h>
@@ -33,24 +35,6 @@
 /* Macro to do nothing, but avoid compiler warnings about empty if bodies &c */
 #define NOP (void)0
 
-#if 0
-/* These macros work, but we don't need fgetpos/fsetpos - fseek/ftell
- * have equivalent functionality for files < 2Gbytes in size */
-
-/* unix-style libraries may only have ftell/fseek, not fgetpos/fsetpos */
-#ifndef HAVE_FGETPOS
-/* ftell returns -1L on error; fgetpos returns non-zero on error */
-# define fgetpos(FH, P) ((*(P) = ftell(FH)) == -1L)
-# define fsetpos(FH, P) fseek(FH, *(P), SEEK_SET)
-typedef long int fpos_t;
-#endif
-#endif
-
-/* deals with Borland TurboC & maybe others */
-#if (!defined(CLOCKS_PER_SEC) && defined(CLK_TCK))
-# define CLOCKS_PER_SEC CLK_TCK
-#endif
-
 /* Return max/min of two numbers (if not defined already) */
 /* NB Bad news if X or Y has side-effects... */
 #ifndef max
@@ -60,7 +44,7 @@ typedef long int fpos_t;
 # define min(X, Y) ((X) < (Y) ? (X) : (Y))
 #endif
 
-/* Some C libraries define this in math.h */
+/* M_PI, etc may be defined in math.h */
 #ifndef M_PI
 # ifdef PI /* MSVC defines PI IIRC */
 #  define M_PI PI
@@ -110,17 +94,19 @@ typedef long int fpos_t;
 /* print a line followed by a newline char to a file */
 #define fputsnl(SZ, FH) BLK(fputs((SZ), (FH)); putc('\n', (FH));)
 #define sqrd(X) ((X) * (X))        /* macro to square things */
-#define radius(X, Y) sqrt(sqrd((double)(X)) + sqrd((double)(Y))) /* euclidean distance */
+
+/* 2D Euclidean distance */
+#ifndef HAVE_HYPOT
+# define hypot(X, Y) sqrt(sqrd((double)(X)) + sqrd((double)(Y)))
+#endif
 #define rad(X) ((M_PI / 180.0) * (X))  /* convert from degrees to radians */
 #define deg(X) ((180.0 / M_PI) * (X))  /* convert from radians to degrees */
 
 /* macro to convert argument to a string literal */
-#define STRING(X) _STRING(X)
-#define _STRING(X) #X
+#define STRING(X) STRING_(X)
+#define STRING_(X) #X
 
 #include "osdepend.h"
-#include "filename.h"
-#include "message.h"
 
 #ifndef WORDS_BIGENDIAN
 extern INT16_T useful_w16;
@@ -131,10 +117,11 @@ extern INT32_T useful_w32;
 # define get16(FH) (fread(&useful_w16, 2, 1, (FH)), useful_w16)
 # define get32(FH) (fread(&useful_w32, 4, 1, (FH)), useful_w32)
 #else
-void FAR useful_put16(INT16_T, FILE *);
-void FAR useful_put32(INT32_T, FILE *);
-INT16_T FAR useful_get16(FILE *);
-INT32_T FAR useful_get32(FILE *);
+/* FIXME: why are these "Far"? */
+void Far useful_put16(INT16_T, FILE *);
+void Far useful_put32(INT32_T, FILE *);
+INT16_T Far useful_get16(FILE *);
+INT32_T Far useful_get32(FILE *);
 
 # define put16(W, FH) useful_put16(W, FH)
 # define put32(W, FH) useful_put32(W, FH)
